# Standard library imports
from base64 import b64encode, b64decode
import binascii
from datetime import datetime, timedelta, timezone
from json import loads
import logging
from threading import Lock
from typing import List
from urllib.parse import urlparse, parse_qs, urlencode
from cryptography.hazmat.primitives.serialization import load_der_private_key

# 3rd party library imports
from requests import codes, post, Response

# Local imports
from .api.developer_key_management import CreateSigningKeyRequest
from .date_time import DateTime
from .error import Error
from .multiton import Multiton
from .reference import Reference


class ApplicationToken(metaclass=Multiton):
    """
    Initialize, refresh and supply an eBay OAuth ***application*** token.

    This is a facade for the oath module.
    """

    __slots__ = (
        "_lock",
        "_sandbox",
        "_application_scopes",
        "_application_token",
        "_oauth2api_inst",
    )

    def __init__(
        self,
        sandbox: bool,
        client_id: str or None = None,
        client_secret: str or None = None,
        ru_name: str or None = None,
        application_scopes: List[str] or None = None,
    ) -> None:
        """
        :param sandbox (bool, required): The system to use, True for Sandbox/Testing and False for Production.

        # application/client credentials are optional if you don't make API calls that need an application/client token
        :param client_id (str, optional):
        :param client_secret (str, optional):
        :param ru_name (str, optional):
        :param application_scopes (list, optional):
        :return: None (None)
        """
        self._lock = Lock()
        # The Multiton decorator wraps this initializer with a thread lock; it is safe to skip using self._lock.

        self._sandbox = sandbox

        # application/client credentials
        self._application_scopes = application_scopes

        # token object storage
        self._application_token = None

        # instantiate low-level oauth api utilities
        self._oauth2api_inst = _OAuth2Api(sandbox, client_id, client_secret, ru_name)

    def get(self) -> str:
        """
        Get an eBay Application Token.

        :return: token (str)
        """
        with self._lock:
            if self._application_scopes is None:
                self._determine_application_scopes()

            if self._application_token is None:
                self._refresh_application()
            elif (
                self._application_token.token_expiry.replace(tzinfo=timezone.utc)
                <= DateTime.now()
            ):
                self._refresh_application()

            token = self._application_token.access_token

        return token

    def _determine_application_scopes(self) -> None:
        """
        Determine the application scopes that are currently allowed.

        return: None (None)
        """
        if self._sandbox:
            # permission is always granted for all
            scopes = list(Reference.get_application_scopes().keys())
        else:
            scopes = []
            for scope in Reference.get_application_scopes():
                token_application = self._oauth2api_inst.get_application_token([scope])
                if token_application.error is None:
                    if token_application.access_token is not None:
                        if len(token_application.access_token) > 0:
                            scopes.append(scope)

        self._application_scopes = scopes

    def _refresh_application(self) -> None:
        """
        Refresh the eBay Application Token and update all that comes with it.

        :return None (None)
        """
        token_application = self._oauth2api_inst.get_application_token(
            self._application_scopes
        )
        if token_application.error is not None:
            reason = "token_application.error " + token_application.error
            raise Error(number=96001, reason=reason)
        if (token_application.access_token is None) or (
            len(token_application.access_token) == 0
        ):
            raise Error(
                number=96002, reason="token_application.access_token is missing."
            )

        self._application_token = token_application


class UserToken(metaclass=Multiton):
    """
    Initialize, refresh and supply an eBay OAuth ***user*** token.
    This is a facade for the oath module.
    """

    __slots__ = (
        "_lock",
        "_sandbox",
        "_user_id",
        "_user_password",
        "_user_scopes",
        "_user_token",
        "_oauth2api_inst",
        "_user_refresh_token",
        "_user_refresh_token_expiry",
        "_allow_get_user_consent",
    )

    def __init__(
        self,
        sandbox: bool,
        client_id: str or None = None,
        client_secret: str or None = None,
        ru_name: str or None = None,
        user_id: str or None = None,
        user_password: str or None = None,
        user_scopes: List[str] or None = None,
        user_refresh_token: str or None = None,
        user_refresh_token_expiry: str or None = None,
    ) -> None:
        """
        :param sandbox (bool, required): The system to use, True for Sandbox/Testing and False for Production.

        # application/client credentials, optional if you don't make API calls that need an application/client token
        :param client_id (str, optional):
        :param client_secret (str, optional):
        :param ru_name (str, optional):

        # user credentials, optional if you don't make API calls that need a user token
        :param user_id (str, optional):
        :param user_password (str, optional):
        :param user_scopes (str, optional):

        # user token supply, optional if you don't mind a Chrome browser opening when getting a user token
        :param user_refresh_token (str, optional):
        :param user_refresh_token_expiry (str, optional):

        :return None (None)
        """

        self._lock = Lock()
        # The Multiton decorator wraps this initializer with a thread lock; it is safe to skip using self._lock.

        self._sandbox = sandbox

        self._user_id = user_id
        self._user_password = user_password
        self._user_scopes = user_scopes

        # token object storage
        self._user_token = None

        # instantiate low-level oauth api utilities
        self._oauth2api_inst = _OAuth2Api(sandbox, client_id, client_secret, ru_name)

        # possible user token with expiry
        self._user_refresh_token = user_refresh_token
        self._user_refresh_token_expiry = None
        self._allow_get_user_consent = True
        if user_refresh_token is not None:
            if user_refresh_token_expiry is not None:
                self._user_refresh_token_expiry = DateTime.from_string(
                    user_refresh_token_expiry
                )
                user_refresh_token_expiry = DateTime.from_string(
                    user_refresh_token_expiry
                )
                if user_refresh_token_expiry <= DateTime.now():
                    raise Error(
                        number=96025,
                        reason="The supplied user refresh token has an expired date.",
                        detail=f"Supplied {user_refresh_token_expiry}. Now {DateTime.now()}.",
                    )

                self._allow_get_user_consent = False
                if self._user_scopes is None:
                    self._determine_user_scopes()
                try:
                    self._user_refresh_token_expiry = user_refresh_token_expiry
                except Error as error:
                    raise Error(
                        number=96003,
                        reason="user_refresh_token_expiry value error",
                        detail=error.reason,
                    )
                self._user_refresh_token = _OAuthToken(
                    refresh_token=self._user_refresh_token,
                    refresh_token_expiry=self._user_refresh_token_expiry,
                )
            else:
                raise Error(
                    number=96004,
                    reason="Must supply refresh token expiry with refresh token!",
                )

    def get(self) -> str:
        """Get an eBay User Access Token.

        : return token (str)
        """
        with self._lock:
            if self._user_scopes is None:
                self._determine_user_scopes()

            if self._user_token is None:
                self._refresh_user()
            elif (
                self._user_token.token_expiry.replace(tzinfo=timezone.utc)
                <= DateTime.now()
            ):
                self._refresh_user()

            token = self._user_token.access_token
        return token

    def _determine_user_scopes(self) -> None:
        """
        Determine the user access scopes that are currently allowed.
        :return None (None)
        """
        if self._sandbox:
            # permission is always granted for all
            scopes = list(Reference.get_user_scopes().keys())
        else:
            # these are always granted, many more are possible
            scopes = [
                "https://api.ebay.com/oauth/api_scope",
                "https://api.ebay.com/oauth/api_scope/sell.inventory",
                "https://api.ebay.com/oauth/api_scope/sell.marketing",
                "https://api.ebay.com/oauth/api_scope/sell.account",
                "https://api.ebay.com/oauth/api_scope/sell.fulfillment",
            ]
        self._user_scopes = scopes

    def _refresh_user(self) -> None:
        """
        Refresh the eBay User Access Token and update all that comes with it.
        If we don't have a current refresh token, run the authorization flow.

        :return None (None)
        """
        if self._user_refresh_token is None:
            # We don't have a refresh token; run authorization flow
            self._authorization_flow()

        elif self._user_refresh_token.refresh_token_expiry <= DateTime.now():
            # The refresh token has expired; run authorization flow
            self._authorization_flow()

        else:
            # Exchange our still current refresh token for a new user access token
            self._refresh_user_token()

    def _authorization_flow(self) -> None:
        """
        Get an authorization code by running the authorization_flow, and
        then exchange that for a refresh token (which also contains a
        user token).

        :return None (None)
        """
        if not self._allow_get_user_consent:
            raise Error(
                number=96005, reason="Getting user consent via browser disabled"
            )

        sign_in_url = self._oauth2api_inst.generate_user_authorization_url(
            self._user_scopes
        )
        if sign_in_url is None:
            raise Error(number=96006, reason="sign_in_url is None.")

        code = self._get_authorization_code(sign_in_url)

        refresh_token = self._oauth2api_inst.exchange_code_for_access_token(code)

        if refresh_token.access_token is None:
            raise Error(number=96007, reason="refresh_token.access_token is None.")
        if len(refresh_token.access_token) == 0:
            raise Error(
                number=96008, reason="refresh_token.access_token is of length zero."
            )

        # Split token into refresh token and user token parts
        self._user_refresh_token = _OAuthToken(
            refresh_token=refresh_token.refresh_token,
            refresh_token_expiry=refresh_token.refresh_token_expiry,
        )
        self._user_token = _OAuthToken(
            access_token=refresh_token.access_token,
            token_expiry=refresh_token.token_expiry,
        )

        # Give the user a helpful suggestion.
        expiry = DateTime.to_string(refresh_token.refresh_token_expiry)
        message = (
            f"Edit to your ebay_rest.json file to avoid the browser pop-up.\n"
            f"For the user with an email or username of {self._user_id}.\n"
            f'"refresh_token": "{refresh_token.refresh_token}"\n'
            f'"refresh_token_expiry": "{expiry}"'
        )
        logging.info(message)

    def _get_authorization_code(self, sign_in_url: str) -> str:
        """Run the authorization flow in order to get an authorization code,
        which can subsequently be exchanged for a refresh (and user) token.

        :param sign_in_url (str): The redirect URL for gaining user consent.
        :return code (str): Authorization code.
        """
        try:
            from playwright.sync_api import (
                sync_playwright,
                Error as PlaywrightError,
                TimeoutError as PlaywrightTimeoutError,
            )
        except ModuleNotFoundError:
            reason = f"Supply an 'eBay user token' or install the COMPLETE variant of ebay_rest. Refer to the README.md at https://github.com/matecsaj/ebay_rest."  # noqa: E501
            logging.critical(reason)
            raise Error(number=96019, reason=reason)
        else:

            # Begin with all Playwright resources set to None.
            playwright = browser = context = page = None

            try:
                # Initialize Playwright
                playwright = sync_playwright().start()
                # Launch the browser in a non-headless mode
                browser = playwright.chromium.launch(headless=False)
                # Open a new browser context (equivalent to an incognito window)
                context = browser.new_context()
                # Open a new page within the context
                page = context.new_page()

                # load the initial page
                page.goto(sign_in_url)

                # fill in the username then click continue
                # sometimes a captcha appears that the user must fill in
                page.wait_for_selector("input[name='userid']").type(self._user_id)
                page.wait_for_selector("#signin-continue-btn").click()

                # fill in the password then submit
                page.wait_for_selector("input[name='pass']").type(self._user_password)
                page.wait_for_selector("#sgnBt").click()

                # Several things may happen on the way to the last page and how we need to respond varies.
                seconds = 0
                while seconds < 30:

                    selectors = [
                        "#passkeys-cancel-btn",  # "Simplify your sign-in" prompt; skip it for now
                        "#submit",  # "I agree" prompt; agree
                    ]
                    for selector in selectors:
                        try:
                            page.wait_for_selector(selector, timeout=1)
                        except PlaywrightTimeoutError:
                            pass
                        else:
                            seconds = 0  # reset our timer when a click happens

                    # have we reached the final page?
                    try:
                        page.wait_for_selector("#thnk-wrap", timeout=1000)  # 1 second
                    except PlaywrightTimeoutError:
                        # no! Perhaps we need to wait while the user enters their 2FA code.
                        seconds += 1  # increment out timer
                    else:
                        # yes!
                        # parse the url's query parameters into a dictionary
                        parsed_url = urlparse(page.url)
                        query_params = parse_qs(parsed_url.query)

                        # Check isAuthSuccessful is true, if present
                        is_auth_successful = False
                        if "isAuthSuccessful" in query_params:
                            if "true" == query_params["isAuthSuccessful"][0]:
                                is_auth_successful = True
                        if is_auth_successful:
                            if "code" in query_params:
                                return query_params["code"][0]
                            else:
                                raise Error(
                                    number=96009, reason="Unable to obtain code."
                                )
                        else:
                            reason = f"Authorization failed, check userid & password:{self._user_id} {self._user_password}"
                            raise Error(number=96010, reason=reason)

                raise Error(
                    number=96027,
                    reason="Last page not found.",
                    detail="Has eBay's website changed?",
                )

            except PlaywrightError as e:
                detail = str(e)
                # Check for Chromium not installed error
                if "BrowserType.launch" in detail:
                    number = 96029
                    reason = "Chromium is not installed. Run `playwright install chromium` to install it."
                # Check for "element not found" error
                elif "No node found for selector" in detail:
                    number = 96015
                    reason = "An element was not found on the page. Has eBay's website changed?"
                else:
                    number = (96030,)
                    reason = ("Playwright encountered an unexpected error.",)
                logging.critical(reason)
                raise Error(number=number, reason=reason, detail=detail)

            except PlaywrightTimeoutError:
                raise Error(
                    number=96016, reason="Timeout.", detail="Slow computer or Internet?"
                )

            finally:
                # Ensure that Playwright resources are cleaned up; don't reorder.
                if page:
                    page.close()
                if context:
                    context.close()
                if browser:
                    browser.close()
                if playwright:
                    playwright.stop()

    def _refresh_user_token(self) -> None:
        """
        Exchange a refresh token for a current user token.
        :return: None (None)
        """
        user_token = self._oauth2api_inst.get_access_token(
            self._user_refresh_token.refresh_token, self._user_scopes
        )

        if user_token.access_token is None:
            raise Error(
                number=96011, reason=user_token.token_response["error_description"]
            )
        if len(user_token.access_token) == 0:
            raise Error(
                number=96012, reason="user_token.access_token is of length zero."
            )

        self._user_token = user_token


class _OAuthToken(object):
    __slots__ = (
        "access_token",
        "token_expiry",
        "refresh_token",
        "refresh_token_expiry",
        "error",
        "token_response",
    )

    def __init__(
        self,
        error: str or None = None,
        access_token: str or None = None,
        refresh_token: str or None = None,
        refresh_token_expiry: datetime or None = None,
        token_expiry: datetime or None = None,
    ):
        """
        :param error (str, optional):
        :param access_token (str, optional):
        :param refresh_token (str, optional):
        :param refresh_token_expiry (datetime, optional): datetime in UTC
        :param token_expiry (datetime, optional): datetime in UTC
        :return None (None)
        """
        self.access_token = access_token
        self.token_expiry = token_expiry
        self.refresh_token = refresh_token
        self.refresh_token_expiry = refresh_token_expiry
        self.error = error
        self.token_response = None

    def __str__(self) -> str:
        """
        :return token_str: (str)
        """
        token_str = "{"
        if self.error is not None:
            token_str += '"error": "' + self.error + '"'
        elif self.access_token is not None:
            token_str += (
                '"access_token": "'
                + self.access_token
                + '", "expires_in": "'
                + self.token_expiry.strftime("%Y-%m-%dT%H:%M:%S:%f")
                + '"'
            )
            if self.refresh_token is not None:
                token_str += (
                    ', "refresh_token": "'
                    + self.refresh_token
                    + '", "refresh_token_expire_in": "'
                    + self.refresh_token_expiry.strftime("%Y-%m-%dT%H:%M:%S:%f")
                    + '"'
                )
        token_str += "}"
        return token_str


class _OAuth2Api:
    __slots__ = "_sandbox", "_client_id", "_client_secret", "_ru_name"

    def __init__(self, sandbox: bool, client_id: str, client_secret: str, ru_name: str):
        """Initialize OAuth2Api instance

        :param sandbox (bool, required):
        :param client_id (str, required):
        :param client_secret (str, required):
        :param ru_name (str, required):
        """
        self._sandbox = sandbox

        # application/client
        self._client_id = client_id
        self._client_secret = client_secret
        self._ru_name = ru_name

    def generate_user_authorization_url(
        self, scopes: List[str], state: str or None = None
    ) -> str:
        """
        :param scopes (list(str)), required)
        :param state (str, optional)
        """
        param = {
            "client_id": self._client_id,
            "redirect_uri": self._ru_name,
            "response_type": "code",
            "prompt": "login",
            "scope": " ".join(scopes),
        }

        if state is not None:
            param.update({"state": state})

        query = urlencode(param)
        if self._sandbox:
            web_endpoint = "https://auth.sandbox.ebay.com/oauth2/authorize"
        else:
            web_endpoint = "https://auth.ebay.com/oauth2/authorize"
        return web_endpoint + "?" + query

    def get_application_token(self, scopes: List[str]) -> _OAuthToken:
        """
        Makes call for application token and stores result in credential object.
        :param scopes (list(str), required)
        :return credential_object (_OAuthToken)
        """
        logging.debug("Trying to get a new application access token ... ")
        headers = self._generate_request_headers()
        body = self._generate_application_request_body(scopes)
        api_endpoint = self._get_endpoint()
        resp = post(api_endpoint, data=body, headers=headers)
        content = loads(resp.content)
        token = _OAuthToken()

        return self._finish(resp, token, content)

    def exchange_code_for_access_token(self, code: str) -> _OAuthToken:
        """
        :param code (str, required)
        :return credential_object (_OAuthToken)
        """
        logging.debug("Trying to get a new user access token ... ")

        headers = self._generate_request_headers()
        body = self._generate_oauth_request_body(code)
        api_endpoint = self._get_endpoint()
        resp = post(api_endpoint, data=body, headers=headers)

        content = loads(resp.content)
        token = _OAuthToken()

        if resp.status_code == codes.ok:
            token.refresh_token = content["refresh_token"]
            token.refresh_token_expiry = (
                datetime.now(timezone.utc)
                + timedelta(seconds=int(content["refresh_token_expires_in"]))
                - timedelta(minutes=5)
            )

        return self._finish(resp, token, content)

    def get_access_token(self, refresh_token: str, scopes: List[str]) -> _OAuthToken:
        """
        refresh token call
        :param refresh_token (str, required)
        :param scopes (list(str), required)
        :return credential_object (_OAuthToken)
        """
        logging.debug("Trying to get a new user access token ... ")

        headers = self._generate_request_headers()
        body = self._generate_refresh_request_body(scopes, refresh_token)
        api_endpoint = self._get_endpoint()
        resp = post(api_endpoint, data=body, headers=headers)
        content = loads(resp.content)
        token = _OAuthToken()
        token.token_response = content

        return self._finish(resp, token, content)

    def _get_endpoint(self) -> str:
        """
        :return url (str)
        :return:
        """
        if self._sandbox:
            return "https://api.sandbox.ebay.com/identity/v1/oauth2/token"
        else:
            return "https://api.ebay.com/identity/v1/oauth2/token"

    def _generate_request_headers(self) -> dict:
        """
        :return headers (dict)
        """
        b64_encoded_credential = b64encode(
            (self._client_id + ":" + self._client_secret).encode()
        )
        headers = {
            "Content-Type": "application/x-www-form-urlencoded",
            "Authorization": "Basic " + b64_encoded_credential.decode(),
        }
        return headers

    def _generate_application_request_body(self, scopes: List[str]) -> dict:
        """
        :param scopes list(str)
        :return body (dict)
        """
        body = {
            "grant_type": "client_credentials",
            "redirect_uri": self._ru_name,
            "scope": " ".join(scopes),
        }
        return body

    @staticmethod
    def _generate_refresh_request_body(scopes: List[str], refresh_token: str) -> dict:
        """
        :param scopes (list(str), required):
        :param refresh_token (str, required):
        :return body (dict):
        """
        if refresh_token is None:
            raise Error(
                number=96013,
                reason="credential object does not contain refresh_token and/or scopes",
            )
        body = {
            "grant_type": "refresh_token",
            "refresh_token": refresh_token,
            "scope": " ".join(scopes),
        }
        return body

    def _generate_oauth_request_body(self, code: str) -> dict:
        """
        :param code (str):
        :return body (dict):
        """
        body = {
            "grant_type": "authorization_code",
            "redirect_uri": self._ru_name,
            "code": code,
        }
        return body

    @staticmethod
    def _finish(resp: Response, token: _OAuthToken, content: dict) -> _OAuthToken:
        """
        :param resp (Response, required):
        :param token (_OAuthToken:, required):
        :param content (dict, required):
        :return:
        """
        if resp.status_code == codes.ok:
            token.access_token = content["access_token"]
            token.token_expiry = (
                datetime.now(timezone.utc)
                + timedelta(seconds=int(content["expires_in"]))
                - timedelta(minutes=5)
            )
        else:
            token.error = str(resp.status_code)
            key = "error_description"
            if key in content:
                token.error += ": " + content[key]

        return token


class KeyPairToken(metaclass=Multiton):
    """An eBay private/public key pair created using
    the eBay Key Management API.
    Digital signature credentials are optional if you don't make API
    calls that need a public/private key pair.
    """

    __slots__ = (
        "_lock",
        "_creation_time",
        "_expiration_time",
        "_jwe",
        "_private_key",
        "_public_key",
        "_signing_key_cipher",
        "_signing_key_id",
    )

    def __init__(
        self,
        creation_time: int or None = None,
        expiration_time: int or None = None,
        jwe: str or None = None,
        private_key: str or None = None,
        public_key: str or None = None,
        signing_key_cipher: str or None = None,
        signing_key_id: str or None = None,
    ) -> None:
        """
        # eBay key pair details
        :param creation_time (int, optional):
        :param expiration_time (int, optional):
        :param jwe (str, optional):
        :param private_key (str, optional):
        :param public_key (str, optional):
        :param signing_key_cipher (str, optional):
        :param signing_key_id (str, optional):
        :return: None (None)
        """
        self._lock = Lock()
        # The Multiton decorator wraps this initializer with a thread lock; it is safe to skip using self._lock.
        self._creation_time = (
            DateTime.from_string(creation_time) if creation_time else None
        )
        self._expiration_time = (
            DateTime.from_string(expiration_time) if expiration_time else None
        )
        self._jwe = jwe
        self._private_key = private_key
        self._public_key = public_key
        self._signing_key_cipher = signing_key_cipher
        self._signing_key_id = signing_key_id
        if self._signing_key_cipher:
            self._signing_key_cipher = self._signing_key_cipher.upper()
            if self._signing_key_cipher != "ED25519":
                raise Error(
                    number=96020,
                    reason="Digital Signatures key pair invalid cipher",
                    detail="Only the ED25519 cipher is supported by ebay_rest",
                )

    def key_dict(self):
        """Get the public and private key ready for a request"""
        try:
            decoded = b64decode(self._private_key)
        except binascii.Error as e:
            raise Error(
                number=96028,
                reason="The key is not a valid base64 encoded string.",
                detail=str(e),
                cause=e,
            )
        else:
            pk = load_der_private_key(decoded, password=None)
            return {"jwe": self._jwe, "private_key": pk}

    def _current_key_sufficient(self) -> bool:
        """Check if the current key pair is sufficient to call get_signing_key.

        Returns True if so, else False

        :return bool
        """
        return self._private_key and self._signing_key_id

    def _current_key_in_date(self) -> bool or None:
        """Check if the current key pair has a date and, if it does, if
        the key pair is in date.
        Returns True if the key has an expiry date and is in date.
        Returns False if the key has and expiry date and is expired.
        Returns None if the key has no expiry date.

        :return bool
        """
        if self._expiration_time:
            return (DateTime.now() - timedelta(seconds=90)) < self._expiration_time
        else:
            return None

    def _has_valid_key(self, api) -> bool:
        """
        Check we have enough information to request a new key pair.
        Load the new key pair, and check it is valid.

        :param api (API): A valid API instance that can be used to make
            a KeyManagementAPI call
        :return dict
        """

        if not self._current_key_sufficient():
            return False

        self._get_signing_key(api)

        if not self._current_key_in_date:
            # An expired key must be replaced
            return False

        return True

    def _ensure_key_pair(self, api) -> None:
        """
        Ensures a valid key pair is available.

        :param api (API): A valid API instance that can be used to make
            a KeyManagementAPI call
        :return None (None)

        - If the key is out of date, create a new key pair.
        - If the necessary details for making an API call and the expiration
            time are provided, do nothing (assume the key pair is OK).
        - If the private key and the signing key ID are provided but details
          are not complete, load the key info.
        """

        in_date = self._expiration_time and (
            (DateTime.now() - timedelta(seconds=90)) < self._expiration_time
        )

        if self._expiration_time and not in_date:
            # An expired key must be replaced
            self._create_key_pair(api)

        elif self._expiration_time and self._jwe and self._private_key:
            # If we have enough information to try an API call plus the
            # (in date) expiration time, assume the details are valid
            return

        elif self._current_key_sufficient():
            # We can reload the key as long as we have the private key
            # and the signing key ID
            self._get_signing_key(api)
            if (DateTime.now() - timedelta(seconds=90)) > self._expiration_time:
                # If the loaded key is out of date, raise an error
                self._create_key_pair(api)
                raise Error(
                    number=96021,
                    reason="Expired key pair",
                    detail="Key {} is expired".format(self._signing_key_id),
                )

        else:
            # Raise an error if no good key pair provided
            raise Error(
                number=96022,
                reason="No valid key pair",
                detail="Generate a new key pair",
            )

    def _get_signing_key(self, api) -> None:
        """
        Use the eBay Key Pair Management API to load an existing eBay
        public/private key pair.

        :param api (API): A valid API instance that can be used to make
            a KeyManagementAPI call
        :return None (None)
        """
        try:
            key = api.developer_key_management_get_signing_key(
                signing_key_id=self._signing_key_id
            )
        except Error as e:
            raise Error(
                number=96023,
                reason="Key pair not found",
                detail="Key {} not found".format(self._signing_key_id),
                cause=e,
            )
        else:
            self._save_key(key)

    def _create_key_pair(self, api) -> None:
        """
        Use the eBay Key Pair Management API to generate a new eBay
        public/private key pair

        :param api (API): A valid API instance that can be used to make
            a KeyManagementAPI call
        :return None (None)
        """
        body = CreateSigningKeyRequest(signing_key_cipher="ED25519")
        try:
            key = api.developer_key_management_create_signing_key(
                content_type="application/json", body=body
            )
        except Error as e:
            raise Error(
                number=96024,
                reason="Failed to create key pair",
                detail="Failed to create key pair: {}".format(e),
                cause=e,
            )
        else:
            self._save_key(key)

    def _load_key(self) -> dict:
        """
        Load the saved key dictionary.
        """
        creation_time = DateTime.to_string(self._creation_time)
        expiration_time = DateTime.to_string(self._expiration_time)
        return {
            "creation_time": creation_time,
            "expiration_time": expiration_time,
            "jwe": self._jwe,
            "private_key": self._private_key,
            "public_key": self._public_key,
            "signing_key_cipher": self._signing_key_cipher,
            "signing_key_id": self._signing_key_id,
        }

    def _save_key(self, key) -> None:
        """
        Save a loaded key dictionary to this KeyPairToken
        :param key (dict): A dict of key properties
        """
        self._creation_time = datetime.fromtimestamp(
            key["creation_time"], tz=timezone.utc
        )
        self._expiration_time = datetime.fromtimestamp(
            key["expiration_time"], tz=timezone.utc
        )
        self._jwe = key["jwe"]
        if key["private_key"]:
            # Only write private key if supplied (i.e. just created)
            self._private_key = key["private_key"]
        self._public_key = key["public_key"]
        self._signing_key_cipher = key["signing_key_cipher"]
        self._signing_key_id = key["signing_key_id"]



class Error(Exception):
    """
    Custom exception class for all errors raised by this library.

    This class encapsulates errors with unique error numbers, detailed messages,
    and optional contextual information, ensuring consistent exception handling
    and backward compatibility.

    Attributes:
        number (int): A unique natural number code identifying the error.
        reason (str): A short description of the error's cause.
        detail (str, optional): Additional details about the error, if available.
        __cause__ (Exception, optional): The original exception causing this error,
                                         if applicable.

    Usage Notes for Library Users:
        - Handle exceptions based on the `number` attribute for precise error handling.
        - Error numbers are stable and unlikely to change.
        - Use the `detail` attribute and original exception (`__cause__`) for debugging.

    Usage Notes for Library Maintainers:
        - Assign error numbers following the format of a two-digit prefix (module identifier)
          and a three-digit suffix (error identifier). Example: `96024`.
        - Avoid repurposing existing error numbers.
        - Preserve the original exception trace when re-raising by passing it via the `cause` argument.
        - Maintain backward compatibility when modifying this class.

    Methods:
        as_dict: Returns a dictionary representation of the error for structured logging.
        log: Logs the error message to the provided logger instance.

    Example:
        try:
            external_function()
        except ExternalLibraryError as e:
            raise Error(
                number=12345,
                reason="Failed during external library call",
                detail="Ensure API credentials are valid.",
                cause=e
            )
    """

    __slots__ = "number", "reason", "detail", "__cause__"

    def __init__(
        self, number: int, reason: str, detail: str = None, cause: Exception = None
    ) -> None:
        """
        Instantiate a new Error object.

        Args:
            number (int): A unique natural number code.
            reason (str): A short description of the reason for the error.
            detail (str, optional): Additional details about the failure.
            cause (Exception, optional): The original exception causing this error.

        Returns:
            None
        """
        super().__init__()
        self.number = number
        self.reason = reason
        self.detail = detail
        self.__cause__ = cause  # Store the original exception if provided

    def __str__(self) -> str:
        """
        Returns the error message as a string.

        Returns:
            str: A formatted error message including the number, reason, detail,
                 and cause (if applicable).
        """
        message = f"Error number {self.number}. {self.reason or ''} {self.detail or ''}".strip()
        if self.__cause__:
            message += f" (caused by {type(self.__cause__).__name__}: {self.__cause__})"
        return message

    def as_dict(self) -> dict:
        """
        Returns a dictionary representation of the error.

        Returns:
            dict: A dictionary with the error's attributes.
        """
        return {
            "number": self.number,
            "reason": self.reason,
            "detail": self.detail,
            "cause": str(self.__cause__) if self.__cause__ else None,
        }

    def log(self, logger) -> None:
        """
        Logs the error message using the provided logger instance.

        Args:
            logger: A logger instance with a `error` method.

        Returns:
            None
        """
        logger.error(self.__str__())



# coding: utf-8

"""
    Metadata API

    The Metadata API has operations that retrieve configuration details pertaining to the different eBay marketplaces. In addition to marketplace information, the API also has operations that get information that helps sellers list items on eBay.  # noqa: E501

    OpenAPI spec version: v1.9.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class PaginationInput(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'limit': 'int',
        'offset': 'int'
    }

    attribute_map = {
        'limit': 'limit',
        'offset': 'offset'
    }

    def __init__(self, limit=None, offset=None):  # noqa: E501
        """PaginationInput - a model defined in Swagger"""  # noqa: E501
        self._limit = None
        self._offset = None
        self.discriminator = None
        if limit is not None:
            self.limit = limit
        if offset is not None:
            self.offset = offset

    @property
    def limit(self):
        """Gets the limit of this PaginationInput.  # noqa: E501

        The max number of items, from the current result set, returned on a single page.<br><br><span class=\"tablenote\"><b>Note:</b> For <b>getProductCompatibilities</b>, the max value is 100. If no <b>limit</b> is specified, this field defaults to the max value.</span>  # noqa: E501

        :return: The limit of this PaginationInput.  # noqa: E501
        :rtype: int
        """
        return self._limit

    @limit.setter
    def limit(self, limit):
        """Sets the limit of this PaginationInput.

        The max number of items, from the current result set, returned on a single page.<br><br><span class=\"tablenote\"><b>Note:</b> For <b>getProductCompatibilities</b>, the max value is 100. If no <b>limit</b> is specified, this field defaults to the max value.</span>  # noqa: E501

        :param limit: The limit of this PaginationInput.  # noqa: E501
        :type: int
        """

        self._limit = limit

    @property
    def offset(self):
        """Gets the offset of this PaginationInput.  # noqa: E501

        The number of items that will be skipped in the result set before returning the first item in the paginated response.<br><br>Combine <b>offset</b> with <b>limit</b> to control the items returned in the response. For example, if you supply an offset of 10 and a limit of 20, the first page of the response contains items 11-30 from the complete result set.<br><br><b>Default:</b> 0  # noqa: E501

        :return: The offset of this PaginationInput.  # noqa: E501
        :rtype: int
        """
        return self._offset

    @offset.setter
    def offset(self, offset):
        """Sets the offset of this PaginationInput.

        The number of items that will be skipped in the result set before returning the first item in the paginated response.<br><br>Combine <b>offset</b> with <b>limit</b> to control the items returned in the response. For example, if you supply an offset of 10 and a limit of 20, the first page of the response contains items 11-30 from the complete result set.<br><br><b>Default:</b> 0  # noqa: E501

        :param offset: The offset of this PaginationInput.  # noqa: E501
        :type: int
        """

        self._offset = offset

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(PaginationInput, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, PaginationInput):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Buy Feed API

    The Feed API provides the ability to download TSV_GZIP feed files containing eBay items and an hourly snapshot file for a specific category, date, and marketplace.<br /><br />In addition to the API, there is an open-source Feed SDK written in Java that downloads, combines files into a single file when needed, and unzips the entire feed file. It also lets you specify field filters to curate the items in the file.  # noqa: E501

    OpenAPI spec version: v1.1.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class FeedTypeSearchResponse(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'feed_types': 'list[FeedType]',
        'href': 'str',
        'limit': 'int',
        'next': 'str',
        'total': 'int'
    }

    attribute_map = {
        'feed_types': 'feedTypes',
        'href': 'href',
        'limit': 'limit',
        'next': 'next',
        'total': 'total'
    }

    def __init__(self, feed_types=None, href=None, limit=None, next=None, total=None):  # noqa: E501
        """FeedTypeSearchResponse - a model defined in Swagger"""  # noqa: E501
        self._feed_types = None
        self._href = None
        self._limit = None
        self._next = None
        self._total = None
        self.discriminator = None
        if feed_types is not None:
            self.feed_types = feed_types
        if href is not None:
            self.href = href
        if limit is not None:
            self.limit = limit
        if next is not None:
            self.next = next
        if total is not None:
            self.total = total

    @property
    def feed_types(self):
        """Gets the feed_types of this FeedTypeSearchResponse.  # noqa: E501

        An array of the feed types that match the search criteria.  # noqa: E501

        :return: The feed_types of this FeedTypeSearchResponse.  # noqa: E501
        :rtype: list[FeedType]
        """
        return self._feed_types

    @feed_types.setter
    def feed_types(self, feed_types):
        """Sets the feed_types of this FeedTypeSearchResponse.

        An array of the feed types that match the search criteria.  # noqa: E501

        :param feed_types: The feed_types of this FeedTypeSearchResponse.  # noqa: E501
        :type: list[FeedType]
        """

        self._feed_types = feed_types

    @property
    def href(self):
        """Gets the href of this FeedTypeSearchResponse.  # noqa: E501

        The URL to to the current set of results.  # noqa: E501

        :return: The href of this FeedTypeSearchResponse.  # noqa: E501
        :rtype: str
        """
        return self._href

    @href.setter
    def href(self, href):
        """Sets the href of this FeedTypeSearchResponse.

        The URL to to the current set of results.  # noqa: E501

        :param href: The href of this FeedTypeSearchResponse.  # noqa: E501
        :type: str
        """

        self._href = href

    @property
    def limit(self):
        """Gets the limit of this FeedTypeSearchResponse.  # noqa: E501

        The number of records to show in the current response.  # noqa: E501

        :return: The limit of this FeedTypeSearchResponse.  # noqa: E501
        :rtype: int
        """
        return self._limit

    @limit.setter
    def limit(self, limit):
        """Sets the limit of this FeedTypeSearchResponse.

        The number of records to show in the current response.  # noqa: E501

        :param limit: The limit of this FeedTypeSearchResponse.  # noqa: E501
        :type: int
        """

        self._limit = limit

    @property
    def next(self):
        """Gets the next of this FeedTypeSearchResponse.  # noqa: E501

        You can use this URL to retrieve the next page of results beyond those displayed on the page if there are more results that match the search criteria.  # noqa: E501

        :return: The next of this FeedTypeSearchResponse.  # noqa: E501
        :rtype: str
        """
        return self._next

    @next.setter
    def next(self, next):
        """Sets the next of this FeedTypeSearchResponse.

        You can use this URL to retrieve the next page of results beyond those displayed on the page if there are more results that match the search criteria.  # noqa: E501

        :param next: The next of this FeedTypeSearchResponse.  # noqa: E501
        :type: str
        """

        self._next = next

    @property
    def total(self):
        """Gets the total of this FeedTypeSearchResponse.  # noqa: E501

        The total number of matches for the search criteria.  # noqa: E501

        :return: The total of this FeedTypeSearchResponse.  # noqa: E501
        :rtype: int
        """
        return self._total

    @total.setter
    def total(self, total):
        """Sets the total of this FeedTypeSearchResponse.

        The total number of matches for the search criteria.  # noqa: E501

        :param total: The total of this FeedTypeSearchResponse.  # noqa: E501
        :type: int
        """

        self._total = total

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(FeedTypeSearchResponse, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, FeedTypeSearchResponse):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Metadata API

    The Metadata API has operations that retrieve configuration details pertaining to the different eBay marketplaces. In addition to marketplace information, the API also has operations that get information that helps sellers list items on eBay.  # noqa: E501

    OpenAPI spec version: v1.9.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class ItemConditionDescriptor(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'condition_descriptor_constraint': 'ItemConditionDescriptorConstraint',
        'condition_descriptor_help_text': 'str',
        'condition_descriptor_id': 'str',
        'condition_descriptor_name': 'str',
        'condition_descriptor_values': 'list[ItemConditionDescriptorValue]'
    }

    attribute_map = {
        'condition_descriptor_constraint': 'conditionDescriptorConstraint',
        'condition_descriptor_help_text': 'conditionDescriptorHelpText',
        'condition_descriptor_id': 'conditionDescriptorId',
        'condition_descriptor_name': 'conditionDescriptorName',
        'condition_descriptor_values': 'conditionDescriptorValues'
    }

    def __init__(self, condition_descriptor_constraint=None, condition_descriptor_help_text=None, condition_descriptor_id=None, condition_descriptor_name=None, condition_descriptor_values=None):  # noqa: E501
        """ItemConditionDescriptor - a model defined in Swagger"""  # noqa: E501
        self._condition_descriptor_constraint = None
        self._condition_descriptor_help_text = None
        self._condition_descriptor_id = None
        self._condition_descriptor_name = None
        self._condition_descriptor_values = None
        self.discriminator = None
        if condition_descriptor_constraint is not None:
            self.condition_descriptor_constraint = condition_descriptor_constraint
        if condition_descriptor_help_text is not None:
            self.condition_descriptor_help_text = condition_descriptor_help_text
        if condition_descriptor_id is not None:
            self.condition_descriptor_id = condition_descriptor_id
        if condition_descriptor_name is not None:
            self.condition_descriptor_name = condition_descriptor_name
        if condition_descriptor_values is not None:
            self.condition_descriptor_values = condition_descriptor_values

    @property
    def condition_descriptor_constraint(self):
        """Gets the condition_descriptor_constraint of this ItemConditionDescriptor.  # noqa: E501


        :return: The condition_descriptor_constraint of this ItemConditionDescriptor.  # noqa: E501
        :rtype: ItemConditionDescriptorConstraint
        """
        return self._condition_descriptor_constraint

    @condition_descriptor_constraint.setter
    def condition_descriptor_constraint(self, condition_descriptor_constraint):
        """Sets the condition_descriptor_constraint of this ItemConditionDescriptor.


        :param condition_descriptor_constraint: The condition_descriptor_constraint of this ItemConditionDescriptor.  # noqa: E501
        :type: ItemConditionDescriptorConstraint
        """

        self._condition_descriptor_constraint = condition_descriptor_constraint

    @property
    def condition_descriptor_help_text(self):
        """Gets the condition_descriptor_help_text of this ItemConditionDescriptor.  # noqa: E501

        A description of the condition descriptor that directs a user to its condition descriptor values.<br><br> For example, the help text for <code>Card Condition</code> is <code>Select ungraded condition</code>.  # noqa: E501

        :return: The condition_descriptor_help_text of this ItemConditionDescriptor.  # noqa: E501
        :rtype: str
        """
        return self._condition_descriptor_help_text

    @condition_descriptor_help_text.setter
    def condition_descriptor_help_text(self, condition_descriptor_help_text):
        """Sets the condition_descriptor_help_text of this ItemConditionDescriptor.

        A description of the condition descriptor that directs a user to its condition descriptor values.<br><br> For example, the help text for <code>Card Condition</code> is <code>Select ungraded condition</code>.  # noqa: E501

        :param condition_descriptor_help_text: The condition_descriptor_help_text of this ItemConditionDescriptor.  # noqa: E501
        :type: str
        """

        self._condition_descriptor_help_text = condition_descriptor_help_text

    @property
    def condition_descriptor_id(self):
        """Gets the condition_descriptor_id of this ItemConditionDescriptor.  # noqa: E501

        The unique identification number of a condition descriptor associated with with a <b>conditionDescriptorName</b>. <br><br>For example, <code>40001</code> is the ID for <code>Card Condition</code>.<br><br>These IDs are used in the addItem family of calls of the <b>Trading API</b> to provide condition descriptor names for the item. These IDs are used by the inventoryItem family of calls of the <b>Inventory API</b> to provide condition descriptor names for the item.  # noqa: E501

        :return: The condition_descriptor_id of this ItemConditionDescriptor.  # noqa: E501
        :rtype: str
        """
        return self._condition_descriptor_id

    @condition_descriptor_id.setter
    def condition_descriptor_id(self, condition_descriptor_id):
        """Sets the condition_descriptor_id of this ItemConditionDescriptor.

        The unique identification number of a condition descriptor associated with with a <b>conditionDescriptorName</b>. <br><br>For example, <code>40001</code> is the ID for <code>Card Condition</code>.<br><br>These IDs are used in the addItem family of calls of the <b>Trading API</b> to provide condition descriptor names for the item. These IDs are used by the inventoryItem family of calls of the <b>Inventory API</b> to provide condition descriptor names for the item.  # noqa: E501

        :param condition_descriptor_id: The condition_descriptor_id of this ItemConditionDescriptor.  # noqa: E501
        :type: str
        """

        self._condition_descriptor_id = condition_descriptor_id

    @property
    def condition_descriptor_name(self):
        """Gets the condition_descriptor_name of this ItemConditionDescriptor.  # noqa: E501

        The human-readable label for the condition descriptor associated with the <b>conditionDescriptorID</b>. <br><br>For example, <code>Card Condition</code> is the condition descriptor name for ID <code>40001</code>  # noqa: E501

        :return: The condition_descriptor_name of this ItemConditionDescriptor.  # noqa: E501
        :rtype: str
        """
        return self._condition_descriptor_name

    @condition_descriptor_name.setter
    def condition_descriptor_name(self, condition_descriptor_name):
        """Sets the condition_descriptor_name of this ItemConditionDescriptor.

        The human-readable label for the condition descriptor associated with the <b>conditionDescriptorID</b>. <br><br>For example, <code>Card Condition</code> is the condition descriptor name for ID <code>40001</code>  # noqa: E501

        :param condition_descriptor_name: The condition_descriptor_name of this ItemConditionDescriptor.  # noqa: E501
        :type: str
        """

        self._condition_descriptor_name = condition_descriptor_name

    @property
    def condition_descriptor_values(self):
        """Gets the condition_descriptor_values of this ItemConditionDescriptor.  # noqa: E501

        This array shows the possible values that map to the corresponding <b>conditionDescriptorName</b> values. Constraint information and help text are also shown for each value. <br><br>For example, The ID <code>40001</code> is ID for the condition descriptor <code>card condition</code>. The ID <code>400012</code> is the ID for the <code>Very Good</code> card condition value.  # noqa: E501

        :return: The condition_descriptor_values of this ItemConditionDescriptor.  # noqa: E501
        :rtype: list[ItemConditionDescriptorValue]
        """
        return self._condition_descriptor_values

    @condition_descriptor_values.setter
    def condition_descriptor_values(self, condition_descriptor_values):
        """Sets the condition_descriptor_values of this ItemConditionDescriptor.

        This array shows the possible values that map to the corresponding <b>conditionDescriptorName</b> values. Constraint information and help text are also shown for each value. <br><br>For example, The ID <code>40001</code> is ID for the condition descriptor <code>card condition</code>. The ID <code>400012</code> is the ID for the <code>Very Good</code> card condition value.  # noqa: E501

        :param condition_descriptor_values: The condition_descriptor_values of this ItemConditionDescriptor.  # noqa: E501
        :type: list[ItemConditionDescriptorValue]
        """

        self._condition_descriptor_values = condition_descriptor_values

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(ItemConditionDescriptor, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ItemConditionDescriptor):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Fulfillment API

    Use the Fulfillment API to complete the process of packaging, addressing, handling, and shipping each order on behalf of the seller, in accordance with the payment method and timing specified at checkout.  # noqa: E501

    OpenAPI spec version: v1.20.4
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class AddEvidencePaymentDisputeRequest(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'evidence_type': 'str',
        'files': 'list[FileEvidence]',
        'line_items': 'list[OrderLineItems]'
    }

    attribute_map = {
        'evidence_type': 'evidenceType',
        'files': 'files',
        'line_items': 'lineItems'
    }

    def __init__(self, evidence_type=None, files=None, line_items=None):  # noqa: E501
        """AddEvidencePaymentDisputeRequest - a model defined in Swagger"""  # noqa: E501
        self._evidence_type = None
        self._files = None
        self._line_items = None
        self.discriminator = None
        if evidence_type is not None:
            self.evidence_type = evidence_type
        if files is not None:
            self.files = files
        if line_items is not None:
            self.line_items = line_items

    @property
    def evidence_type(self):
        """Gets the evidence_type of this AddEvidencePaymentDisputeRequest.  # noqa: E501

        This field is used to indicate the type of evidence being provided through one or more evidence files. All evidence files (if more than one) should be associated with the evidence type passed in this field.<br><br>See the <a href=\"/api-docs/sell/fulfillment/types/api:EvidenceTypeEnum\" target=\"_blank \">EvidenceTypeEnum</a> type for the supported evidence types. For implementation help, refer to <a href='https://developer.ebay.com/api-docs/sell/fulfillment/types/api:EvidenceTypeEnum'>eBay API documentation</a>  # noqa: E501

        :return: The evidence_type of this AddEvidencePaymentDisputeRequest.  # noqa: E501
        :rtype: str
        """
        return self._evidence_type

    @evidence_type.setter
    def evidence_type(self, evidence_type):
        """Sets the evidence_type of this AddEvidencePaymentDisputeRequest.

        This field is used to indicate the type of evidence being provided through one or more evidence files. All evidence files (if more than one) should be associated with the evidence type passed in this field.<br><br>See the <a href=\"/api-docs/sell/fulfillment/types/api:EvidenceTypeEnum\" target=\"_blank \">EvidenceTypeEnum</a> type for the supported evidence types. For implementation help, refer to <a href='https://developer.ebay.com/api-docs/sell/fulfillment/types/api:EvidenceTypeEnum'>eBay API documentation</a>  # noqa: E501

        :param evidence_type: The evidence_type of this AddEvidencePaymentDisputeRequest.  # noqa: E501
        :type: str
        """

        self._evidence_type = evidence_type

    @property
    def files(self):
        """Gets the files of this AddEvidencePaymentDisputeRequest.  # noqa: E501

        This array is used to specify one or more evidence files that will become part of a new evidence set associated with a payment dispute. At least one evidence file must be specified in the <strong>files</strong> array.  # noqa: E501

        :return: The files of this AddEvidencePaymentDisputeRequest.  # noqa: E501
        :rtype: list[FileEvidence]
        """
        return self._files

    @files.setter
    def files(self, files):
        """Sets the files of this AddEvidencePaymentDisputeRequest.

        This array is used to specify one or more evidence files that will become part of a new evidence set associated with a payment dispute. At least one evidence file must be specified in the <strong>files</strong> array.  # noqa: E501

        :param files: The files of this AddEvidencePaymentDisputeRequest.  # noqa: E501
        :type: list[FileEvidence]
        """

        self._files = files

    @property
    def line_items(self):
        """Gets the line_items of this AddEvidencePaymentDisputeRequest.  # noqa: E501

        This array identifies the order line item(s) for which the evidence file(s) will be applicable.<br><Br>These values are returned under the <strong>evidenceRequests.lineItems</strong> array in the <a href=\"/api-docs/sell/fulfillment/resources/payment_dispute/methods/getPaymentDispute\" target=\"_blank \">getPaymentDispute</a> response.  # noqa: E501

        :return: The line_items of this AddEvidencePaymentDisputeRequest.  # noqa: E501
        :rtype: list[OrderLineItems]
        """
        return self._line_items

    @line_items.setter
    def line_items(self, line_items):
        """Sets the line_items of this AddEvidencePaymentDisputeRequest.

        This array identifies the order line item(s) for which the evidence file(s) will be applicable.<br><Br>These values are returned under the <strong>evidenceRequests.lineItems</strong> array in the <a href=\"/api-docs/sell/fulfillment/resources/payment_dispute/methods/getPaymentDispute\" target=\"_blank \">getPaymentDispute</a> response.  # noqa: E501

        :param line_items: The line_items of this AddEvidencePaymentDisputeRequest.  # noqa: E501
        :type: list[OrderLineItems]
        """

        self._line_items = line_items

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(AddEvidencePaymentDisputeRequest, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, AddEvidencePaymentDisputeRequest):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Browse API

    The Browse API has the following resources:<ul><li><b>item_summary:</b><br>Allows shoppers to search for specific items by keyword, GTIN, category, charity, product, image, or item aspects and refine the results by using filters, such as aspects, compatibility, and fields values, or UI parameters.</li><li><b>item:</b><br>Allows shoppers to retrieve the details of a specific item or all items in an item group, which is an item with variations such as color and size and check if a product is compatible with the specified item, such as if a specific car is compatible with a specific part.<br><br>This resource also provides a bridge between the eBay legacy APIs, such as the <a href=\"/api-docs/user-guides/static/finding-user-guide-landing.html\" target=\"_blank\">Finding</b>, and the RESTful APIs, which use different formats for the item IDs.</li></ul>The <b>item_summary</b>, <b>search_by_image</b>, and <b>item</b> resource calls require an <a href=\"/api-docs/static/oauth-client-credentials-grant.html\" target=\"_blank\">Application access token</a>.  # noqa: E501

    OpenAPI spec version: v1.19.9
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class RatingHistogram(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'count': 'int',
        'rating': 'str'
    }

    attribute_map = {
        'count': 'count',
        'rating': 'rating'
    }

    def __init__(self, count=None, rating=None):  # noqa: E501
        """RatingHistogram - a model defined in Swagger"""  # noqa: E501
        self._count = None
        self._rating = None
        self.discriminator = None
        if count is not None:
            self.count = count
        if rating is not None:
            self.rating = rating

    @property
    def count(self):
        """Gets the count of this RatingHistogram.  # noqa: E501

        The total number of user ratings that the product has received.  # noqa: E501

        :return: The count of this RatingHistogram.  # noqa: E501
        :rtype: int
        """
        return self._count

    @count.setter
    def count(self, count):
        """Sets the count of this RatingHistogram.

        The total number of user ratings that the product has received.  # noqa: E501

        :param count: The count of this RatingHistogram.  # noqa: E501
        :type: int
        """

        self._count = count

    @property
    def rating(self):
        """Gets the rating of this RatingHistogram.  # noqa: E501

        This is the average rating for the product. As part of a product review, users rate the product. Products are rated from one star (terrible) to five stars (excellent), with each star having a corresponding point value - one star gets 1 point, two stars get 2 points, and so on. If a product had one four-star rating and one five-star rating, its average rating would be <code> 4.5</code>, and this is the value that would appear in this field.  # noqa: E501

        :return: The rating of this RatingHistogram.  # noqa: E501
        :rtype: str
        """
        return self._rating

    @rating.setter
    def rating(self, rating):
        """Sets the rating of this RatingHistogram.

        This is the average rating for the product. As part of a product review, users rate the product. Products are rated from one star (terrible) to five stars (excellent), with each star having a corresponding point value - one star gets 1 point, two stars get 2 points, and so on. If a product had one four-star rating and one five-star rating, its average rating would be <code> 4.5</code>, and this is the value that would appear in this field.  # noqa: E501

        :param rating: The rating of this RatingHistogram.  # noqa: E501
        :type: str
        """

        self._rating = rating

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(RatingHistogram, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, RatingHistogram):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Marketing API

    <p>The <i>Marketing API </i> offers two platforms that sellers can use to promote and advertise their products:</p> <ul><li><b>Promoted Listings</b> is an eBay ad service that lets sellers set up <i>ad campaigns </i> for the products they want to promote. eBay displays the ads in search results and in other marketing modules as <b>SPONSORED</b> listings. If an item in a Promoted Listings campaign sells, the seller is assessed a Promoted Listings fee, which is a seller-specified percentage applied to the sales price. For complete details, refer to the <a href=\"/api-docs/sell/static/marketing/pl-landing.html\">Promoted Listings playbook</a>.</li><li><b>Promotions Manager</b> gives sellers a way to offer discounts on specific items as a way to attract buyers to their inventory. Sellers can set up discounts (such as \"20% off\" and other types of offers) on specific items or on an entire customer order. To further attract buyers, eBay prominently displays promotion <i>teasers</i> throughout buyer flows. For complete details, see <a href=\"/api-docs/sell/static/marketing/promotions-manager.html\">Promotions Manager</a>.</li></ul>  <p><b>Marketing reports</b>, on both the Promoted Listings and Promotions Manager platforms, give sellers information that shows the effectiveness of their marketing strategies. The data gives sellers the ability to review and fine tune their marketing efforts.</p><p><b>Store Email Campaign</b> allows sellers to create and send email campaigns to customers who have signed up to receive their newsletter. For more information on email campaigns, see <a href=\"/api-docs/sell/static/marketing/store-email-campaigns.html#email-campain-types\" target=\"_blank\">Store Email Campaigns</a>.<p class=\"tablenote\"><b>Important!</b> Sellers must have an active eBay Store subscription, and they must accept the <b>Terms and Conditions</b> before they can make requests to these APIs in the Production environment. There are also site-specific listings requirements and restrictions associated with these marketing tools, as listed in the \"requirements and restrictions\" sections for <a href=\"/api-docs/sell/marketing/static/overview.html#PL-requirements\">Promoted Listings</a> and <a href=\"/api-docs/sell/marketing/static/overview.html#PM-requirements\">Promotions Manager</a>.</p> <p>The table below lists all the Marketing API calls grouped by resource.</p>  # noqa: E501

    OpenAPI spec version: v1.22.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class UpdateCampaignRequest(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'audience_codes': 'list[str]',
        'category_id': 'str',
        'category_type': 'str',
        'item_ids': 'list[str]',
        'item_select_mode': 'str',
        'personalized_message': 'str',
        'price_range': 'PriceRange',
        'promotion_id': 'str',
        'promotion_select_mode_enum': 'str',
        'schedule_date': 'str',
        'sort': 'str',
        'subject': 'str'
    }

    attribute_map = {
        'audience_codes': 'audienceCodes',
        'category_id': 'categoryId',
        'category_type': 'categoryType',
        'item_ids': 'itemIds',
        'item_select_mode': 'itemSelectMode',
        'personalized_message': 'personalizedMessage',
        'price_range': 'priceRange',
        'promotion_id': 'promotionId',
        'promotion_select_mode_enum': 'promotionSelectModeEnum',
        'schedule_date': 'scheduleDate',
        'sort': 'sort',
        'subject': 'subject'
    }

    def __init__(self, audience_codes=None, category_id=None, category_type=None, item_ids=None, item_select_mode=None, personalized_message=None, price_range=None, promotion_id=None, promotion_select_mode_enum=None, schedule_date=None, sort=None, subject=None):  # noqa: E501
        """UpdateCampaignRequest - a model defined in Swagger"""  # noqa: E501
        self._audience_codes = None
        self._category_id = None
        self._category_type = None
        self._item_ids = None
        self._item_select_mode = None
        self._personalized_message = None
        self._price_range = None
        self._promotion_id = None
        self._promotion_select_mode_enum = None
        self._schedule_date = None
        self._sort = None
        self._subject = None
        self.discriminator = None
        if audience_codes is not None:
            self.audience_codes = audience_codes
        if category_id is not None:
            self.category_id = category_id
        if category_type is not None:
            self.category_type = category_type
        if item_ids is not None:
            self.item_ids = item_ids
        if item_select_mode is not None:
            self.item_select_mode = item_select_mode
        if personalized_message is not None:
            self.personalized_message = personalized_message
        if price_range is not None:
            self.price_range = price_range
        if promotion_id is not None:
            self.promotion_id = promotion_id
        if promotion_select_mode_enum is not None:
            self.promotion_select_mode_enum = promotion_select_mode_enum
        if schedule_date is not None:
            self.schedule_date = schedule_date
        if sort is not None:
            self.sort = sort
        if subject is not None:
            self.subject = subject

    @property
    def audience_codes(self):
        """Gets the audience_codes of this UpdateCampaignRequest.  # noqa: E501

        An array of audience codes for the audiences of the email campaign. At least one audience code is required. There is no limit to the number of audience codes that may be entered.<br><br><b>Example: </b> if the current email campaign contains <code>\"audienceCodes\": \"code1\", \"code2\"</code> and the user wishes to add an audience code <code>code3</code>, set the <code>audienceCodes</code> value to <code>\"audienceCodes\": \"code1\", \"code2\", \"code3\"</code>.<br><br>To retrieve seller audiences, call <a href=\"/api-docs/sell/marketing/resources/email_campaign/methods/getAudiences\" target=\"_blank\">getAudiences</a>. Use the code values in the response to populate audienceCodes.  # noqa: E501

        :return: The audience_codes of this UpdateCampaignRequest.  # noqa: E501
        :rtype: list[str]
        """
        return self._audience_codes

    @audience_codes.setter
    def audience_codes(self, audience_codes):
        """Sets the audience_codes of this UpdateCampaignRequest.

        An array of audience codes for the audiences of the email campaign. At least one audience code is required. There is no limit to the number of audience codes that may be entered.<br><br><b>Example: </b> if the current email campaign contains <code>\"audienceCodes\": \"code1\", \"code2\"</code> and the user wishes to add an audience code <code>code3</code>, set the <code>audienceCodes</code> value to <code>\"audienceCodes\": \"code1\", \"code2\", \"code3\"</code>.<br><br>To retrieve seller audiences, call <a href=\"/api-docs/sell/marketing/resources/email_campaign/methods/getAudiences\" target=\"_blank\">getAudiences</a>. Use the code values in the response to populate audienceCodes.  # noqa: E501

        :param audience_codes: The audience_codes of this UpdateCampaignRequest.  # noqa: E501
        :type: list[str]
        """

        self._audience_codes = audience_codes

    @property
    def category_id(self):
        """Gets the category_id of this UpdateCampaignRequest.  # noqa: E501

        The unique identifier of either an eBay category or a store category.<br><br>This field is used if a seller wants to apply an email campaign to a specific eBay category or store category. The <b>categoryType</b> determines whether the <b>categoryId</b> value is an eBay category or store category.<br><br>To retrieve eBay categories, use the <a href=\"/devzone/xml/docs/reference/ebay/GetCategories.html\" target=\"_blank\">getCategories</a> or Taxonomy API. To retrieve seller store categories, use the <a href=\"/devzone/xml/docs/reference/ebay/GetStore.html\" target=\"_blank\">getStore</a> call. Use the <b>categoryId</b> value of the desired category from the results as the value in the request.<br><br><b>itemSelectMode</b> must be set to <code>AUTO</code> in order to use a category ID.  # noqa: E501

        :return: The category_id of this UpdateCampaignRequest.  # noqa: E501
        :rtype: str
        """
        return self._category_id

    @category_id.setter
    def category_id(self, category_id):
        """Sets the category_id of this UpdateCampaignRequest.

        The unique identifier of either an eBay category or a store category.<br><br>This field is used if a seller wants to apply an email campaign to a specific eBay category or store category. The <b>categoryType</b> determines whether the <b>categoryId</b> value is an eBay category or store category.<br><br>To retrieve eBay categories, use the <a href=\"/devzone/xml/docs/reference/ebay/GetCategories.html\" target=\"_blank\">getCategories</a> or Taxonomy API. To retrieve seller store categories, use the <a href=\"/devzone/xml/docs/reference/ebay/GetStore.html\" target=\"_blank\">getStore</a> call. Use the <b>categoryId</b> value of the desired category from the results as the value in the request.<br><br><b>itemSelectMode</b> must be set to <code>AUTO</code> in order to use a category ID.  # noqa: E501

        :param category_id: The category_id of this UpdateCampaignRequest.  # noqa: E501
        :type: str
        """

        self._category_id = category_id

    @property
    def category_type(self):
        """Gets the category_type of this UpdateCampaignRequest.  # noqa: E501

        This field must be set when applying an email campaign to a specific eBay category or store category. The enumeration value used indicates which type of category the <b>categoryId</b> belongs to. For implementation help, refer to <a href='https://developer.ebay.com/api-docs/sell/marketing/types/api:CategoryTypeEnum'>eBay API documentation</a>  # noqa: E501

        :return: The category_type of this UpdateCampaignRequest.  # noqa: E501
        :rtype: str
        """
        return self._category_type

    @category_type.setter
    def category_type(self, category_type):
        """Sets the category_type of this UpdateCampaignRequest.

        This field must be set when applying an email campaign to a specific eBay category or store category. The enumeration value used indicates which type of category the <b>categoryId</b> belongs to. For implementation help, refer to <a href='https://developer.ebay.com/api-docs/sell/marketing/types/api:CategoryTypeEnum'>eBay API documentation</a>  # noqa: E501

        :param category_type: The category_type of this UpdateCampaignRequest.  # noqa: E501
        :type: str
        """

        self._category_type = category_type

    @property
    def item_ids(self):
        """Gets the item_ids of this UpdateCampaignRequest.  # noqa: E501

        An array of unique identifiers for the listings displayed in an email campaign. Used if the seller wishes to select the eBay listings in the email campaign rather than have eBay automatically select them.<br><br>Call <a href=\"/DevZone/XML/docs/Reference/eBay/GetSellerList.html#GetSellerList\">getSellerList</a> to retrieve all seller listings. Each <b>Item</b> result contains an <b>ItemID</b> value. Use this value in <b>itemIds</b> to feature that listing.<br><br>The maximum number of <b>itemIds</b> for the <code>COUPON</code> campaign type is 4, and for every other campaign type is 10.<br><br><b>itemSelectMode</b> must be set to <code>MANUAL</code> in order to use this field.  # noqa: E501

        :return: The item_ids of this UpdateCampaignRequest.  # noqa: E501
        :rtype: list[str]
        """
        return self._item_ids

    @item_ids.setter
    def item_ids(self, item_ids):
        """Sets the item_ids of this UpdateCampaignRequest.

        An array of unique identifiers for the listings displayed in an email campaign. Used if the seller wishes to select the eBay listings in the email campaign rather than have eBay automatically select them.<br><br>Call <a href=\"/DevZone/XML/docs/Reference/eBay/GetSellerList.html#GetSellerList\">getSellerList</a> to retrieve all seller listings. Each <b>Item</b> result contains an <b>ItemID</b> value. Use this value in <b>itemIds</b> to feature that listing.<br><br>The maximum number of <b>itemIds</b> for the <code>COUPON</code> campaign type is 4, and for every other campaign type is 10.<br><br><b>itemSelectMode</b> must be set to <code>MANUAL</code> in order to use this field.  # noqa: E501

        :param item_ids: The item_ids of this UpdateCampaignRequest.  # noqa: E501
        :type: list[str]
        """

        self._item_ids = item_ids

    @property
    def item_select_mode(self):
        """Gets the item_select_mode of this UpdateCampaignRequest.  # noqa: E501

        Determines whether listings featured in an email campaign are selected by the seller or by eBay.<br><br>If <b>itemSelectMode</b> is set to <code>AUTO</code>, eBay automatically choses listings based on values set for <b>sort</b>, <b>categoryType</b>, <b>categoryId</b>, and <b>priceRange</b>.<br><br>If <b>itemSelectMode</b> is set to <code>MANUAL</code>, listings are set by the seller by populating the <code>itemIds</code> array.<br><br><span class=\"tablenote\"><b>Note: </b><b>itemSelectMode</b> is always set to <code>AUTO</code> for <code>WELCOME</code> email campaigns.</span> For implementation help, refer to <a href='https://developer.ebay.com/api-docs/sell/marketing/types/api:ItemSelectModeEnum'>eBay API documentation</a>  # noqa: E501

        :return: The item_select_mode of this UpdateCampaignRequest.  # noqa: E501
        :rtype: str
        """
        return self._item_select_mode

    @item_select_mode.setter
    def item_select_mode(self, item_select_mode):
        """Sets the item_select_mode of this UpdateCampaignRequest.

        Determines whether listings featured in an email campaign are selected by the seller or by eBay.<br><br>If <b>itemSelectMode</b> is set to <code>AUTO</code>, eBay automatically choses listings based on values set for <b>sort</b>, <b>categoryType</b>, <b>categoryId</b>, and <b>priceRange</b>.<br><br>If <b>itemSelectMode</b> is set to <code>MANUAL</code>, listings are set by the seller by populating the <code>itemIds</code> array.<br><br><span class=\"tablenote\"><b>Note: </b><b>itemSelectMode</b> is always set to <code>AUTO</code> for <code>WELCOME</code> email campaigns.</span> For implementation help, refer to <a href='https://developer.ebay.com/api-docs/sell/marketing/types/api:ItemSelectModeEnum'>eBay API documentation</a>  # noqa: E501

        :param item_select_mode: The item_select_mode of this UpdateCampaignRequest.  # noqa: E501
        :type: str
        """

        self._item_select_mode = item_select_mode

    @property
    def personalized_message(self):
        """Gets the personalized_message of this UpdateCampaignRequest.  # noqa: E501

        The body of the email campaign. Accepts HTML and CSS. The maximum length is 1000 characters  # noqa: E501

        :return: The personalized_message of this UpdateCampaignRequest.  # noqa: E501
        :rtype: str
        """
        return self._personalized_message

    @personalized_message.setter
    def personalized_message(self, personalized_message):
        """Sets the personalized_message of this UpdateCampaignRequest.

        The body of the email campaign. Accepts HTML and CSS. The maximum length is 1000 characters  # noqa: E501

        :param personalized_message: The personalized_message of this UpdateCampaignRequest.  # noqa: E501
        :type: str
        """

        self._personalized_message = personalized_message

    @property
    def price_range(self):
        """Gets the price_range of this UpdateCampaignRequest.  # noqa: E501


        :return: The price_range of this UpdateCampaignRequest.  # noqa: E501
        :rtype: PriceRange
        """
        return self._price_range

    @price_range.setter
    def price_range(self, price_range):
        """Sets the price_range of this UpdateCampaignRequest.


        :param price_range: The price_range of this UpdateCampaignRequest.  # noqa: E501
        :type: PriceRange
        """

        self._price_range = price_range

    @property
    def promotion_id(self):
        """Gets the promotion_id of this UpdateCampaignRequest.  # noqa: E501

        The ID of the discount used for an email campaign if the <b>emailCampaignType</b> is set to <code>COUPON</code>, <code>SALE_EVENT</code>, or <code>ORDER_DISCOUNT</code>, and <b>promotionSelectModeEnum</b> is set to <code>MANUAL</code>.<br><br>To find a discount, call <a href=\"/api-docs/sell/marketing/resources/promotion/methods/getPromotions\" target=\"_blank\">getPromotions</a> to retrieve a list of the seller's discounts. Use the <b>promotionId</b> from an individual <b>PromotionDetail</b> result for the request.  # noqa: E501

        :return: The promotion_id of this UpdateCampaignRequest.  # noqa: E501
        :rtype: str
        """
        return self._promotion_id

    @promotion_id.setter
    def promotion_id(self, promotion_id):
        """Sets the promotion_id of this UpdateCampaignRequest.

        The ID of the discount used for an email campaign if the <b>emailCampaignType</b> is set to <code>COUPON</code>, <code>SALE_EVENT</code>, or <code>ORDER_DISCOUNT</code>, and <b>promotionSelectModeEnum</b> is set to <code>MANUAL</code>.<br><br>To find a discount, call <a href=\"/api-docs/sell/marketing/resources/promotion/methods/getPromotions\" target=\"_blank\">getPromotions</a> to retrieve a list of the seller's discounts. Use the <b>promotionId</b> from an individual <b>PromotionDetail</b> result for the request.  # noqa: E501

        :param promotion_id: The promotion_id of this UpdateCampaignRequest.  # noqa: E501
        :type: str
        """

        self._promotion_id = promotion_id

    @property
    def promotion_select_mode_enum(self):
        """Gets the promotion_select_mode_enum of this UpdateCampaignRequest.  # noqa: E501

        The selection mode for the discount used. If set to <code>AUTO</code>, eBay will choose the discount to include in the email campaign. If set to <code>MANUAL</code>, the seller must specify the discount in the <b>promotionId</b> field.<br><br>This field is required if the <b>emailCampaignType</b> is set to <code>COUPON</code>, <code>SALE_EVENT</code>, or <code>ORDER_DISCOUNT</code>. For implementation help, refer to <a href='https://developer.ebay.com/api-docs/sell/marketing/types/api:PromotionSelectModeEnum'>eBay API documentation</a>  # noqa: E501

        :return: The promotion_select_mode_enum of this UpdateCampaignRequest.  # noqa: E501
        :rtype: str
        """
        return self._promotion_select_mode_enum

    @promotion_select_mode_enum.setter
    def promotion_select_mode_enum(self, promotion_select_mode_enum):
        """Sets the promotion_select_mode_enum of this UpdateCampaignRequest.

        The selection mode for the discount used. If set to <code>AUTO</code>, eBay will choose the discount to include in the email campaign. If set to <code>MANUAL</code>, the seller must specify the discount in the <b>promotionId</b> field.<br><br>This field is required if the <b>emailCampaignType</b> is set to <code>COUPON</code>, <code>SALE_EVENT</code>, or <code>ORDER_DISCOUNT</code>. For implementation help, refer to <a href='https://developer.ebay.com/api-docs/sell/marketing/types/api:PromotionSelectModeEnum'>eBay API documentation</a>  # noqa: E501

        :param promotion_select_mode_enum: The promotion_select_mode_enum of this UpdateCampaignRequest.  # noqa: E501
        :type: str
        """

        self._promotion_select_mode_enum = promotion_select_mode_enum

    @property
    def schedule_date(self):
        """Gets the schedule_date of this UpdateCampaignRequest.  # noqa: E501

        The date and time that the email campaign newsletter will be sent, given in UTC format. Example: 2023-05-20T03:13:35Z<br><br>This field should be used if the seller wishes to send the email campaign on a future date. If no <b>scheduleDate</b> is set, the email campaign will send once it is created or updated.  # noqa: E501

        :return: The schedule_date of this UpdateCampaignRequest.  # noqa: E501
        :rtype: str
        """
        return self._schedule_date

    @schedule_date.setter
    def schedule_date(self, schedule_date):
        """Sets the schedule_date of this UpdateCampaignRequest.

        The date and time that the email campaign newsletter will be sent, given in UTC format. Example: 2023-05-20T03:13:35Z<br><br>This field should be used if the seller wishes to send the email campaign on a future date. If no <b>scheduleDate</b> is set, the email campaign will send once it is created or updated.  # noqa: E501

        :param schedule_date: The schedule_date of this UpdateCampaignRequest.  # noqa: E501
        :type: str
        """

        self._schedule_date = schedule_date

    @property
    def sort(self):
        """Gets the sort of this UpdateCampaignRequest.  # noqa: E501

        The sort rule is used to display the listings featured in the email campaign.<br><br>Sort rules are only used if <b>itemSelectMode</b> is set to <code>AUTO</code>. If <b>itemSelectMode</b> is <code>MANUAL</code>, listings are displayed in the order in which they are listed in the <b>itemIds</b> array.<br><br>The following sort rules are available:<ul><li><code>ENDING_FIRST</code> displays listings by ending date, from soonest to latest.</li><li><code>LOWEST_PRICED</code> displays listings by price, from lowest to highest.</li><li><code>HIGHEST_PRICED</code> displays listings by price, from highest to lowest.</li><li><code>NEWLY_LISTED</code> displays listings by date listed, with the newest first.</li></ul><br><br>The default sort rule is <code>NEWLY_LISTED</code>. For implementation help, refer to <a href='https://developer.ebay.com/api-docs/sell/marketing/types/api:ItemSortEnum'>eBay API documentation</a>  # noqa: E501

        :return: The sort of this UpdateCampaignRequest.  # noqa: E501
        :rtype: str
        """
        return self._sort

    @sort.setter
    def sort(self, sort):
        """Sets the sort of this UpdateCampaignRequest.

        The sort rule is used to display the listings featured in the email campaign.<br><br>Sort rules are only used if <b>itemSelectMode</b> is set to <code>AUTO</code>. If <b>itemSelectMode</b> is <code>MANUAL</code>, listings are displayed in the order in which they are listed in the <b>itemIds</b> array.<br><br>The following sort rules are available:<ul><li><code>ENDING_FIRST</code> displays listings by ending date, from soonest to latest.</li><li><code>LOWEST_PRICED</code> displays listings by price, from lowest to highest.</li><li><code>HIGHEST_PRICED</code> displays listings by price, from highest to lowest.</li><li><code>NEWLY_LISTED</code> displays listings by date listed, with the newest first.</li></ul><br><br>The default sort rule is <code>NEWLY_LISTED</code>. For implementation help, refer to <a href='https://developer.ebay.com/api-docs/sell/marketing/types/api:ItemSortEnum'>eBay API documentation</a>  # noqa: E501

        :param sort: The sort of this UpdateCampaignRequest.  # noqa: E501
        :type: str
        """

        self._sort = sort

    @property
    def subject(self):
        """Gets the subject of this UpdateCampaignRequest.  # noqa: E501

        The email campaign subject. The maximum length is 70 characters.  # noqa: E501

        :return: The subject of this UpdateCampaignRequest.  # noqa: E501
        :rtype: str
        """
        return self._subject

    @subject.setter
    def subject(self, subject):
        """Sets the subject of this UpdateCampaignRequest.

        The email campaign subject. The maximum length is 70 characters.  # noqa: E501

        :param subject: The subject of this UpdateCampaignRequest.  # noqa: E501
        :type: str
        """

        self._subject = subject

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(UpdateCampaignRequest, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, UpdateCampaignRequest):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Metadata API

    The Metadata API has operations that retrieve configuration details pertaining to the different eBay marketplaces. In addition to marketplace information, the API also has operations that get information that helps sellers list items on eBay.  # noqa: E501

    OpenAPI spec version: v1.9.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class PropertyNamesResponsePropertyNames(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'property_display_name': 'str',
        'property_name': 'str',
        'property_name_metadata': 'PropertyNamesResponsePropertyNameMetadata'
    }

    attribute_map = {
        'property_display_name': 'propertyDisplayName',
        'property_name': 'propertyName',
        'property_name_metadata': 'propertyNameMetadata'
    }

    def __init__(self, property_display_name=None, property_name=None, property_name_metadata=None):  # noqa: E501
        """PropertyNamesResponsePropertyNames - a model defined in Swagger"""  # noqa: E501
        self._property_display_name = None
        self._property_name = None
        self._property_name_metadata = None
        self.discriminator = None
        if property_display_name is not None:
            self.property_display_name = property_display_name
        if property_name is not None:
            self.property_name = property_name
        if property_name_metadata is not None:
            self.property_name_metadata = property_name_metadata

    @property
    def property_display_name(self):
        """Gets the property_display_name of this PropertyNamesResponsePropertyNames.  # noqa: E501

        The display name of a property. This is the localized name of the compatible property.  # noqa: E501

        :return: The property_display_name of this PropertyNamesResponsePropertyNames.  # noqa: E501
        :rtype: str
        """
        return self._property_display_name

    @property_display_name.setter
    def property_display_name(self, property_display_name):
        """Sets the property_display_name of this PropertyNamesResponsePropertyNames.

        The display name of a property. This is the localized name of the compatible property.  # noqa: E501

        :param property_display_name: The property_display_name of this PropertyNamesResponsePropertyNames.  # noqa: E501
        :type: str
        """

        self._property_display_name = property_display_name

    @property
    def property_name(self):
        """Gets the property_name of this PropertyNamesResponsePropertyNames.  # noqa: E501

        The canonical name of a property. This value is used as part of the name-value pairs used to specify compatibility.  # noqa: E501

        :return: The property_name of this PropertyNamesResponsePropertyNames.  # noqa: E501
        :rtype: str
        """
        return self._property_name

    @property_name.setter
    def property_name(self, property_name):
        """Sets the property_name of this PropertyNamesResponsePropertyNames.

        The canonical name of a property. This value is used as part of the name-value pairs used to specify compatibility.  # noqa: E501

        :param property_name: The property_name of this PropertyNamesResponsePropertyNames.  # noqa: E501
        :type: str
        """

        self._property_name = property_name

    @property
    def property_name_metadata(self):
        """Gets the property_name_metadata of this PropertyNamesResponsePropertyNames.  # noqa: E501


        :return: The property_name_metadata of this PropertyNamesResponsePropertyNames.  # noqa: E501
        :rtype: PropertyNamesResponsePropertyNameMetadata
        """
        return self._property_name_metadata

    @property_name_metadata.setter
    def property_name_metadata(self, property_name_metadata):
        """Sets the property_name_metadata of this PropertyNamesResponsePropertyNames.


        :param property_name_metadata: The property_name_metadata of this PropertyNamesResponsePropertyNames.  # noqa: E501
        :type: PropertyNamesResponsePropertyNameMetadata
        """

        self._property_name_metadata = property_name_metadata

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(PropertyNamesResponsePropertyNames, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, PropertyNamesResponsePropertyNames):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Notification API

    The eBay Notification API enables management of the entire end-to-end eBay notification experience by allowing users to:<ul><li>Browse for supported notification topics and retrieve topic details</li><li>Create, configure, and manage notification destination endpoints</li><li>Configure, manage, and test notification subscriptions</li><li>Process eBay notifications and verify the integrity of the message payload</li></ul>  # noqa: E501

    OpenAPI spec version: v1.6.1
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class PublicKey(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'algorithm': 'str',
        'digest': 'str',
        'key': 'str'
    }

    attribute_map = {
        'algorithm': 'algorithm',
        'digest': 'digest',
        'key': 'key'
    }

    def __init__(self, algorithm=None, digest=None, key=None):  # noqa: E501
        """PublicKey - a model defined in Swagger"""  # noqa: E501
        self._algorithm = None
        self._digest = None
        self._key = None
        self.discriminator = None
        if algorithm is not None:
            self.algorithm = algorithm
        if digest is not None:
            self.digest = digest
        if key is not None:
            self.key = key

    @property
    def algorithm(self):
        """Gets the algorithm of this PublicKey.  # noqa: E501

        The algorithm associated with the public key that is returned, such as Elliptic Curve Digital Signature Algorithm (ECDSA).  # noqa: E501

        :return: The algorithm of this PublicKey.  # noqa: E501
        :rtype: str
        """
        return self._algorithm

    @algorithm.setter
    def algorithm(self, algorithm):
        """Sets the algorithm of this PublicKey.

        The algorithm associated with the public key that is returned, such as Elliptic Curve Digital Signature Algorithm (ECDSA).  # noqa: E501

        :param algorithm: The algorithm of this PublicKey.  # noqa: E501
        :type: str
        """

        self._algorithm = algorithm

    @property
    def digest(self):
        """Gets the digest of this PublicKey.  # noqa: E501

        The digest associated with the public key that is returned, such as Secure Hash Algorithm 1 (SHA1).  # noqa: E501

        :return: The digest of this PublicKey.  # noqa: E501
        :rtype: str
        """
        return self._digest

    @digest.setter
    def digest(self, digest):
        """Sets the digest of this PublicKey.

        The digest associated with the public key that is returned, such as Secure Hash Algorithm 1 (SHA1).  # noqa: E501

        :param digest: The digest of this PublicKey.  # noqa: E501
        :type: str
        """

        self._digest = digest

    @property
    def key(self):
        """Gets the key of this PublicKey.  # noqa: E501

        The public key that is returned for the specified key ID.<br><br>This value is used to validate the eBay push notification message payload.  # noqa: E501

        :return: The key of this PublicKey.  # noqa: E501
        :rtype: str
        """
        return self._key

    @key.setter
    def key(self, key):
        """Sets the key of this PublicKey.

        The public key that is returned for the specified key ID.<br><br>This value is used to validate the eBay push notification message payload.  # noqa: E501

        :param key: The key of this PublicKey.  # noqa: E501
        :type: str
        """

        self._key = key

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(PublicKey, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, PublicKey):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Browse API

    The Browse API has the following resources:<ul><li><b>item_summary:</b><br>Allows shoppers to search for specific items by keyword, GTIN, category, charity, product, image, or item aspects and refine the results by using filters, such as aspects, compatibility, and fields values, or UI parameters.</li><li><b>item:</b><br>Allows shoppers to retrieve the details of a specific item or all items in an item group, which is an item with variations such as color and size and check if a product is compatible with the specified item, such as if a specific car is compatible with a specific part.<br><br>This resource also provides a bridge between the eBay legacy APIs, such as the <a href=\"/api-docs/user-guides/static/finding-user-guide-landing.html\" target=\"_blank\">Finding</b>, and the RESTful APIs, which use different formats for the item IDs.</li></ul>The <b>item_summary</b>, <b>search_by_image</b>, and <b>item</b> resource calls require an <a href=\"/api-docs/static/oauth-client-credentials-grant.html\" target=\"_blank\">Application access token</a>.  # noqa: E501

    OpenAPI spec version: v1.19.9
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class AuthenticityGuaranteeProgram(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'description': 'str',
        'terms_web_url': 'str'
    }

    attribute_map = {
        'description': 'description',
        'terms_web_url': 'termsWebUrl'
    }

    def __init__(self, description=None, terms_web_url=None):  # noqa: E501
        """AuthenticityGuaranteeProgram - a model defined in Swagger"""  # noqa: E501
        self._description = None
        self._terms_web_url = None
        self.discriminator = None
        if description is not None:
            self.description = description
        if terms_web_url is not None:
            self.terms_web_url = terms_web_url

    @property
    def description(self):
        """Gets the description of this AuthenticityGuaranteeProgram.  # noqa: E501

        An indication that the item is qualified for the Authenticity Guarantee program.  # noqa: E501

        :return: The description of this AuthenticityGuaranteeProgram.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this AuthenticityGuaranteeProgram.

        An indication that the item is qualified for the Authenticity Guarantee program.  # noqa: E501

        :param description: The description of this AuthenticityGuaranteeProgram.  # noqa: E501
        :type: str
        """

        self._description = description

    @property
    def terms_web_url(self):
        """Gets the terms_web_url of this AuthenticityGuaranteeProgram.  # noqa: E501

        The URL to the Authenticity Guarantee program terms of use.  # noqa: E501

        :return: The terms_web_url of this AuthenticityGuaranteeProgram.  # noqa: E501
        :rtype: str
        """
        return self._terms_web_url

    @terms_web_url.setter
    def terms_web_url(self, terms_web_url):
        """Sets the terms_web_url of this AuthenticityGuaranteeProgram.

        The URL to the Authenticity Guarantee program terms of use.  # noqa: E501

        :param terms_web_url: The terms_web_url of this AuthenticityGuaranteeProgram.  # noqa: E501
        :type: str
        """

        self._terms_web_url = terms_web_url

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(AuthenticityGuaranteeProgram, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, AuthenticityGuaranteeProgram):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Media API

    The <b>Media API</b> lets sellers to create, upload, and retrieve files, including:<ul><li>videos</li><li>documents (for GPSR regulations)</li></ul>  # noqa: E501

    OpenAPI spec version: v1_beta.2.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class DocumentResponse(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'document_id': 'str',
        'document_metadata': 'DocumentMetadata',
        'document_status': 'str',
        'document_type': 'str',
        'languages': 'list[str]'
    }

    attribute_map = {
        'document_id': 'documentId',
        'document_metadata': 'documentMetadata',
        'document_status': 'documentStatus',
        'document_type': 'documentType',
        'languages': 'languages'
    }

    def __init__(self, document_id=None, document_metadata=None, document_status=None, document_type=None, languages=None):  # noqa: E501
        """DocumentResponse - a model defined in Swagger"""  # noqa: E501
        self._document_id = None
        self._document_metadata = None
        self._document_status = None
        self._document_type = None
        self._languages = None
        self.discriminator = None
        if document_id is not None:
            self.document_id = document_id
        if document_metadata is not None:
            self.document_metadata = document_metadata
        if document_status is not None:
            self.document_status = document_status
        if document_type is not None:
            self.document_type = document_type
        if languages is not None:
            self.languages = languages

    @property
    def document_id(self):
        """Gets the document_id of this DocumentResponse.  # noqa: E501

        The unique ID of the document.  # noqa: E501

        :return: The document_id of this DocumentResponse.  # noqa: E501
        :rtype: str
        """
        return self._document_id

    @document_id.setter
    def document_id(self, document_id):
        """Sets the document_id of this DocumentResponse.

        The unique ID of the document.  # noqa: E501

        :param document_id: The document_id of this DocumentResponse.  # noqa: E501
        :type: str
        """

        self._document_id = document_id

    @property
    def document_metadata(self):
        """Gets the document_metadata of this DocumentResponse.  # noqa: E501


        :return: The document_metadata of this DocumentResponse.  # noqa: E501
        :rtype: DocumentMetadata
        """
        return self._document_metadata

    @document_metadata.setter
    def document_metadata(self, document_metadata):
        """Sets the document_metadata of this DocumentResponse.


        :param document_metadata: The document_metadata of this DocumentResponse.  # noqa: E501
        :type: DocumentMetadata
        """

        self._document_metadata = document_metadata

    @property
    def document_status(self):
        """Gets the document_status of this DocumentResponse.  # noqa: E501

        The status of the document resource.<br><br>Once a document has been uploaded using the <b>uploadDocument</b> method, the <b>documentStatus</b> will be <code>SUBMITTED</code>. The document will then either be accepted or rejected. Only documents with the status of <code>ACCEPTED</code> are available to be added to a listing. For implementation help, refer to <a href='https://developer.ebay.com/api-docs/commerce/media/types/api:DocumentStatusEnum'>eBay API documentation</a>  # noqa: E501

        :return: The document_status of this DocumentResponse.  # noqa: E501
        :rtype: str
        """
        return self._document_status

    @document_status.setter
    def document_status(self, document_status):
        """Sets the document_status of this DocumentResponse.

        The status of the document resource.<br><br>Once a document has been uploaded using the <b>uploadDocument</b> method, the <b>documentStatus</b> will be <code>SUBMITTED</code>. The document will then either be accepted or rejected. Only documents with the status of <code>ACCEPTED</code> are available to be added to a listing. For implementation help, refer to <a href='https://developer.ebay.com/api-docs/commerce/media/types/api:DocumentStatusEnum'>eBay API documentation</a>  # noqa: E501

        :param document_status: The document_status of this DocumentResponse.  # noqa: E501
        :type: str
        """

        self._document_status = document_status

    @property
    def document_type(self):
        """Gets the document_type of this DocumentResponse.  # noqa: E501

        The type of the document uploaded. For example, <code>USER_GUIDE_OR_MANUAL</code>. For implementation help, refer to <a href='https://developer.ebay.com/api-docs/commerce/media/types/api:DocumentTypeEnum'>eBay API documentation</a>  # noqa: E501

        :return: The document_type of this DocumentResponse.  # noqa: E501
        :rtype: str
        """
        return self._document_type

    @document_type.setter
    def document_type(self, document_type):
        """Sets the document_type of this DocumentResponse.

        The type of the document uploaded. For example, <code>USER_GUIDE_OR_MANUAL</code>. For implementation help, refer to <a href='https://developer.ebay.com/api-docs/commerce/media/types/api:DocumentTypeEnum'>eBay API documentation</a>  # noqa: E501

        :param document_type: The document_type of this DocumentResponse.  # noqa: E501
        :type: str
        """

        self._document_type = document_type

    @property
    def languages(self):
        """Gets the languages of this DocumentResponse.  # noqa: E501

        This array shows the language(s) used in the document.  # noqa: E501

        :return: The languages of this DocumentResponse.  # noqa: E501
        :rtype: list[str]
        """
        return self._languages

    @languages.setter
    def languages(self, languages):
        """Sets the languages of this DocumentResponse.

        This array shows the language(s) used in the document.  # noqa: E501

        :param languages: The languages of this DocumentResponse.  # noqa: E501
        :type: list[str]
        """

        self._languages = languages

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(DocumentResponse, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, DocumentResponse):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Inventory API

    The Inventory API is used to create and manage inventory, and then to publish and manage this inventory on an eBay marketplace. There are also methods in this API that will convert eligible, active eBay listings into the Inventory API model.  # noqa: E501

    OpenAPI spec version: 1.18.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class Overrides(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'cut_off_time': 'str',
        'end_date': 'str',
        'start_date': 'str'
    }

    attribute_map = {
        'cut_off_time': 'cutOffTime',
        'end_date': 'endDate',
        'start_date': 'startDate'
    }

    def __init__(self, cut_off_time=None, end_date=None, start_date=None):  # noqa: E501
        """Overrides - a model defined in Swagger"""  # noqa: E501
        self._cut_off_time = None
        self._end_date = None
        self._start_date = None
        self.discriminator = None
        if cut_off_time is not None:
            self.cut_off_time = cut_off_time
        if end_date is not None:
            self.end_date = end_date
        if start_date is not None:
            self.start_date = start_date

    @property
    def cut_off_time(self):
        """Gets the cut_off_time of this Overrides.  # noqa: E501

        This field is used to override the cut-off time(s) specified in the <b>weeklySchedule</b> container. If an order is placed after this time in the specified date or date range, it will be handled by the seller on the following day.<br><br><b>Format:</b> <code>00:00</code>  # noqa: E501

        :return: The cut_off_time of this Overrides.  # noqa: E501
        :rtype: str
        """
        return self._cut_off_time

    @cut_off_time.setter
    def cut_off_time(self, cut_off_time):
        """Sets the cut_off_time of this Overrides.

        This field is used to override the cut-off time(s) specified in the <b>weeklySchedule</b> container. If an order is placed after this time in the specified date or date range, it will be handled by the seller on the following day.<br><br><b>Format:</b> <code>00:00</code>  # noqa: E501

        :param cut_off_time: The cut_off_time of this Overrides.  # noqa: E501
        :type: str
        """

        self._cut_off_time = cut_off_time

    @property
    def end_date(self):
        """Gets the end_date of this Overrides.  # noqa: E501

        The end date of the cut-off time override in <a href=\"https://www.iso.org/iso-8601-date-and-time-format.html \" title=\"https://www.iso.org \" target=\"_blank\">ISO 8601</a> format, which is based on the 24-hour Coordinated Universal Time (UTC) clock.<br><br><span class=\"tablenote\"><b>Note:</b> If the cut-off time override is only for a single day, input the same date in the <b>startDate</b> and <b>endDate</b> fields.</span><br><b>Format:</b> <code>[YYYY]-[MM]-[DD]</code><br><br><b>Example:</b> <code>2024-08-06</code><br><br><span class=\"tablenote\"><b>Note:</b> The time zone for this date is specified from the <b>timeZoneId</b> field. If this field is not used, the time zone will be derived from the provided address.</span>  # noqa: E501

        :return: The end_date of this Overrides.  # noqa: E501
        :rtype: str
        """
        return self._end_date

    @end_date.setter
    def end_date(self, end_date):
        """Sets the end_date of this Overrides.

        The end date of the cut-off time override in <a href=\"https://www.iso.org/iso-8601-date-and-time-format.html \" title=\"https://www.iso.org \" target=\"_blank\">ISO 8601</a> format, which is based on the 24-hour Coordinated Universal Time (UTC) clock.<br><br><span class=\"tablenote\"><b>Note:</b> If the cut-off time override is only for a single day, input the same date in the <b>startDate</b> and <b>endDate</b> fields.</span><br><b>Format:</b> <code>[YYYY]-[MM]-[DD]</code><br><br><b>Example:</b> <code>2024-08-06</code><br><br><span class=\"tablenote\"><b>Note:</b> The time zone for this date is specified from the <b>timeZoneId</b> field. If this field is not used, the time zone will be derived from the provided address.</span>  # noqa: E501

        :param end_date: The end_date of this Overrides.  # noqa: E501
        :type: str
        """

        self._end_date = end_date

    @property
    def start_date(self):
        """Gets the start_date of this Overrides.  # noqa: E501

        The start date of the cut-off time override in <a href=\"https://www.iso.org/iso-8601-date-and-time-format.html \" title=\"https://www.iso.org \" target=\"_blank\">ISO 8601</a> format, which is based on the 24-hour Coordinated Universal Time (UTC) clock.<br><br><span class=\"tablenote\"><b>Note:</b> If the cut-off time override is only for a single day, input the same date in the <b>startDate</b> and <b>endDate</b> fields.</span><br><b>Format:</b> <code>[YYYY]-[MM]-[DD]</code><br><br><b>Example:</b> <code>2024-08-04</code><br><br><span class=\"tablenote\"><b>Note:</b> The time zone for this date is specified from the <b>timeZoneId</b> field. If this field is not used, the time zone will be derived from the provided address.</span>  # noqa: E501

        :return: The start_date of this Overrides.  # noqa: E501
        :rtype: str
        """
        return self._start_date

    @start_date.setter
    def start_date(self, start_date):
        """Sets the start_date of this Overrides.

        The start date of the cut-off time override in <a href=\"https://www.iso.org/iso-8601-date-and-time-format.html \" title=\"https://www.iso.org \" target=\"_blank\">ISO 8601</a> format, which is based on the 24-hour Coordinated Universal Time (UTC) clock.<br><br><span class=\"tablenote\"><b>Note:</b> If the cut-off time override is only for a single day, input the same date in the <b>startDate</b> and <b>endDate</b> fields.</span><br><b>Format:</b> <code>[YYYY]-[MM]-[DD]</code><br><br><b>Example:</b> <code>2024-08-04</code><br><br><span class=\"tablenote\"><b>Note:</b> The time zone for this date is specified from the <b>timeZoneId</b> field. If this field is not used, the time zone will be derived from the provided address.</span>  # noqa: E501

        :param start_date: The start_date of this Overrides.  # noqa: E501
        :type: str
        """

        self._start_date = start_date

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(Overrides, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Overrides):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Store API

    <p>This API provides stores-related resources for third-party developers. These resources let you retrieve basic store information such as store name, description, store url, return store category hierarchy, add,rename,move,delete a single user's eBay store category, and retrieve the processing status of these tasks.</p> <p>The stores resource methods require an access token created with the <a href=\"/api-docs/static/oauth-authorization-code-grant.html\">authorization code grant</a> flow, using one or more scopes from the following list (please check your Application Keys page for a list of OAuth scopes available to your application)</p>  # noqa: E501

    OpenAPI spec version: 1
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class DeleteStoreCategoryRequestType(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'listing_destination_category_id': 'str'
    }

    attribute_map = {
        'listing_destination_category_id': 'listingDestinationCategoryId'
    }

    def __init__(self, listing_destination_category_id=None):  # noqa: E501
        """DeleteStoreCategoryRequestType - a model defined in Swagger"""  # noqa: E501
        self._listing_destination_category_id = None
        self.discriminator = None
        if listing_destination_category_id is not None:
            self.listing_destination_category_id = listing_destination_category_id

    @property
    def listing_destination_category_id(self):
        """Gets the listing_destination_category_id of this DeleteStoreCategoryRequestType.  # noqa: E501

        This field is only needed if the category to be deleted or any of its children categories have one or more active listings. The operation will fail otherwise. All active listings in or under the category to be deleted will be moved to the store category specified in this field.<br>The <a href=\"/api-docs/sell/stores/resources/store/methods/getStoreCategories\"><b>getStoreCategories</b></a> method can be used to retrieve store category IDs.<br><br> <b>Default: OTHER</b> category ID <b>(1)</b> if it's null.  # noqa: E501

        :return: The listing_destination_category_id of this DeleteStoreCategoryRequestType.  # noqa: E501
        :rtype: str
        """
        return self._listing_destination_category_id

    @listing_destination_category_id.setter
    def listing_destination_category_id(self, listing_destination_category_id):
        """Sets the listing_destination_category_id of this DeleteStoreCategoryRequestType.

        This field is only needed if the category to be deleted or any of its children categories have one or more active listings. The operation will fail otherwise. All active listings in or under the category to be deleted will be moved to the store category specified in this field.<br>The <a href=\"/api-docs/sell/stores/resources/store/methods/getStoreCategories\"><b>getStoreCategories</b></a> method can be used to retrieve store category IDs.<br><br> <b>Default: OTHER</b> category ID <b>(1)</b> if it's null.  # noqa: E501

        :param listing_destination_category_id: The listing_destination_category_id of this DeleteStoreCategoryRequestType.  # noqa: E501
        :type: str
        """

        self._listing_destination_category_id = listing_destination_category_id

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(DeleteStoreCategoryRequestType, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, DeleteStoreCategoryRequestType):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Taxonomy API

    Use the Taxonomy API to discover the most appropriate eBay categories under which sellers can offer inventory items for sale, and the most likely categories under which buyers can browse or search for items to purchase. In addition, the Taxonomy API provides metadata about the required and recommended category aspects to include in listings, and also has two operations to retrieve parts compatibility information.  # noqa: E501

    OpenAPI spec version: v1.1.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class ExpiredCategory(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'from_category_id': 'str',
        'to_category_id': 'str'
    }

    attribute_map = {
        'from_category_id': 'fromCategoryId',
        'to_category_id': 'toCategoryId'
    }

    def __init__(self, from_category_id=None, to_category_id=None):  # noqa: E501
        """ExpiredCategory - a model defined in Swagger"""  # noqa: E501
        self._from_category_id = None
        self._to_category_id = None
        self.discriminator = None
        if from_category_id is not None:
            self.from_category_id = from_category_id
        if to_category_id is not None:
            self.to_category_id = to_category_id

    @property
    def from_category_id(self):
        """Gets the from_category_id of this ExpiredCategory.  # noqa: E501

        The unique identifier of the expired eBay leaf category.  # noqa: E501

        :return: The from_category_id of this ExpiredCategory.  # noqa: E501
        :rtype: str
        """
        return self._from_category_id

    @from_category_id.setter
    def from_category_id(self, from_category_id):
        """Sets the from_category_id of this ExpiredCategory.

        The unique identifier of the expired eBay leaf category.  # noqa: E501

        :param from_category_id: The from_category_id of this ExpiredCategory.  # noqa: E501
        :type: str
        """

        self._from_category_id = from_category_id

    @property
    def to_category_id(self):
        """Gets the to_category_id of this ExpiredCategory.  # noqa: E501

        The unique identifier of the currently active eBay leaf category that has replaced the expired leaf category.<br><br><span class=\"tablenote\"><b>Note:</b> More than one <b>fromCategoryID</b> value may map into the same <b>toCategoryID</b> value, as multiple eBay categories may be consolidated into one new, expanded category.</span>  # noqa: E501

        :return: The to_category_id of this ExpiredCategory.  # noqa: E501
        :rtype: str
        """
        return self._to_category_id

    @to_category_id.setter
    def to_category_id(self, to_category_id):
        """Sets the to_category_id of this ExpiredCategory.

        The unique identifier of the currently active eBay leaf category that has replaced the expired leaf category.<br><br><span class=\"tablenote\"><b>Note:</b> More than one <b>fromCategoryID</b> value may map into the same <b>toCategoryID</b> value, as multiple eBay categories may be consolidated into one new, expanded category.</span>  # noqa: E501

        :param to_category_id: The to_category_id of this ExpiredCategory.  # noqa: E501
        :type: str
        """

        self._to_category_id = to_category_id

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(ExpiredCategory, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ExpiredCategory):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Buy Feed API

    The Feed API provides the ability to download TSV_GZIP feed files containing eBay items and an hourly snapshot file for a specific category, date, and marketplace.<br /><br />In addition to the API, there is an open-source Feed SDK written in Java that downloads, combines files into a single file when needed, and unzips the entire feed file. It also lets you specify field filters to curate the items in the file.  # noqa: E501

    OpenAPI spec version: v1.1.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class SupportedFeed(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'authorization_scopes': 'list[str]',
        'constraint': 'Constraint',
        'feed_scope': 'str',
        'look_back': 'TimeDuration',
        'status': 'str',
        'supported_schemas': 'list[SupportedSchema]'
    }

    attribute_map = {
        'authorization_scopes': 'authorizationScopes',
        'constraint': 'constraint',
        'feed_scope': 'feedScope',
        'look_back': 'lookBack',
        'status': 'status',
        'supported_schemas': 'supportedSchemas'
    }

    def __init__(self, authorization_scopes=None, constraint=None, feed_scope=None, look_back=None, status=None, supported_schemas=None):  # noqa: E501
        """SupportedFeed - a model defined in Swagger"""  # noqa: E501
        self._authorization_scopes = None
        self._constraint = None
        self._feed_scope = None
        self._look_back = None
        self._status = None
        self._supported_schemas = None
        self.discriminator = None
        if authorization_scopes is not None:
            self.authorization_scopes = authorization_scopes
        if constraint is not None:
            self.constraint = constraint
        if feed_scope is not None:
            self.feed_scope = feed_scope
        if look_back is not None:
            self.look_back = look_back
        if status is not None:
            self.status = status
        if supported_schemas is not None:
            self.supported_schemas = supported_schemas

    @property
    def authorization_scopes(self):
        """Gets the authorization_scopes of this SupportedFeed.  # noqa: E501

        The oauth authorization scopes which grant access to the feed files.<br /><br />Currently the only applicable authorization scope is <code>https://api.ebay.com/oauth/api_scope/buy.item.feed</code>.<br /><br /><span class=\"tablenote\"><span style=\"color:#004680\"><strong>Note:</strong></span> You can view your application's oauth scopes on the <a href='developer.ebay.com/my/keys'>Application Keys</a> page.</span>  # noqa: E501

        :return: The authorization_scopes of this SupportedFeed.  # noqa: E501
        :rtype: list[str]
        """
        return self._authorization_scopes

    @authorization_scopes.setter
    def authorization_scopes(self, authorization_scopes):
        """Sets the authorization_scopes of this SupportedFeed.

        The oauth authorization scopes which grant access to the feed files.<br /><br />Currently the only applicable authorization scope is <code>https://api.ebay.com/oauth/api_scope/buy.item.feed</code>.<br /><br /><span class=\"tablenote\"><span style=\"color:#004680\"><strong>Note:</strong></span> You can view your application's oauth scopes on the <a href='developer.ebay.com/my/keys'>Application Keys</a> page.</span>  # noqa: E501

        :param authorization_scopes: The authorization_scopes of this SupportedFeed.  # noqa: E501
        :type: list[str]
        """

        self._authorization_scopes = authorization_scopes

    @property
    def constraint(self):
        """Gets the constraint of this SupportedFeed.  # noqa: E501


        :return: The constraint of this SupportedFeed.  # noqa: E501
        :rtype: Constraint
        """
        return self._constraint

    @constraint.setter
    def constraint(self, constraint):
        """Sets the constraint of this SupportedFeed.


        :param constraint: The constraint of this SupportedFeed.  # noqa: E501
        :type: Constraint
        """

        self._constraint = constraint

    @property
    def feed_scope(self):
        """Gets the feed_scope of this SupportedFeed.  # noqa: E501

        Specifies the frequency with which the feed file is made available (<code>HOURLY</code>, <code>DAILY</code>, <code>WEEKLY</code>).<br /><br />Currently only <code>DAILY</code> and <code>HOURLY</code> are supported. For implementation help, refer to <a href='https://developer.ebay.com/api-docs/buy/feed/types/api:FeedScopeEnum'>eBay API documentation</a>  # noqa: E501

        :return: The feed_scope of this SupportedFeed.  # noqa: E501
        :rtype: str
        """
        return self._feed_scope

    @feed_scope.setter
    def feed_scope(self, feed_scope):
        """Sets the feed_scope of this SupportedFeed.

        Specifies the frequency with which the feed file is made available (<code>HOURLY</code>, <code>DAILY</code>, <code>WEEKLY</code>).<br /><br />Currently only <code>DAILY</code> and <code>HOURLY</code> are supported. For implementation help, refer to <a href='https://developer.ebay.com/api-docs/buy/feed/types/api:FeedScopeEnum'>eBay API documentation</a>  # noqa: E501

        :param feed_scope: The feed_scope of this SupportedFeed.  # noqa: E501
        :type: str
        """

        self._feed_scope = feed_scope

    @property
    def look_back(self):
        """Gets the look_back of this SupportedFeed.  # noqa: E501


        :return: The look_back of this SupportedFeed.  # noqa: E501
        :rtype: TimeDuration
        """
        return self._look_back

    @look_back.setter
    def look_back(self, look_back):
        """Sets the look_back of this SupportedFeed.


        :param look_back: The look_back of this SupportedFeed.  # noqa: E501
        :type: TimeDuration
        """

        self._look_back = look_back

    @property
    def status(self):
        """Gets the status of this SupportedFeed.  # noqa: E501

        The status for this feed. One of <code>ACTIVE</code>, <code>PAUSED</code>, or <code>DEPRECATED</code>. For implementation help, refer to <a href='https://developer.ebay.com/api-docs/buy/feed/types/api:FeedStatusEnum'>eBay API documentation</a>  # noqa: E501

        :return: The status of this SupportedFeed.  # noqa: E501
        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, status):
        """Sets the status of this SupportedFeed.

        The status for this feed. One of <code>ACTIVE</code>, <code>PAUSED</code>, or <code>DEPRECATED</code>. For implementation help, refer to <a href='https://developer.ebay.com/api-docs/buy/feed/types/api:FeedStatusEnum'>eBay API documentation</a>  # noqa: E501

        :param status: The status of this SupportedFeed.  # noqa: E501
        :type: str
        """

        self._status = status

    @property
    def supported_schemas(self):
        """Gets the supported_schemas of this SupportedFeed.  # noqa: E501

        An array of the supported <b>Feed</b> API schemas for this feed type.  # noqa: E501

        :return: The supported_schemas of this SupportedFeed.  # noqa: E501
        :rtype: list[SupportedSchema]
        """
        return self._supported_schemas

    @supported_schemas.setter
    def supported_schemas(self, supported_schemas):
        """Sets the supported_schemas of this SupportedFeed.

        An array of the supported <b>Feed</b> API schemas for this feed type.  # noqa: E501

        :param supported_schemas: The supported_schemas of this SupportedFeed.  # noqa: E501
        :type: list[SupportedSchema]
        """

        self._supported_schemas = supported_schemas

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(SupportedFeed, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, SupportedFeed):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other
